
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtGui/QImage>
#include <QtCore/QFileInfo>
#include <QtCore/QDir>
#include <QtCore/QtDebug>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QPainter>
#include <QtGui/QLinearGradient>
#include <QtGui/QRadialGradient>

#include "BackgroundPainter.h"

#include "AppSettings.h"
#include "LogHandler.h"
#include "qimageblitz.h"

#include <cmath>
#ifdef Q_OS_WIN
#define M_PI   3.14159265358979323846   // VC++ does not recognise those (standard) constants
#endif

#ifndef MIN
#define MIN(a,b) ( (a) < (b) ? (a) : (b) )
#endif
#ifndef MAX
#define MAX(a,b) ( (a) > (b) ? (a) : (b) )
#endif
#ifndef ABS
#define ABS(x) ( ( (int)(x) ^ ( (int)(x) >> 31 ) ) - ( (int)(x) >> 31 ) )
#endif

const int previewFactor = 10;

bool operator>(const QSize &s1, const QSize &s2)
{
    return s1.width() > s2.width() || s1.height() > s2.height();
}

// Exponential blur, Jani Huhtanen, 2006
#include <cmath>

template<int aprec, int zprec>
static inline void blurinner(unsigned char *bptr, int &zR, int &zG, int &zB, int &zA, int alpha)
{
    int R,G,B,A;
    R = *bptr;
    G = *(bptr + 1);
    B = *(bptr + 2);
    A = *(bptr + 3);
    zR += ( alpha * ( (R << zprec) - zR ) ) >> aprec;
    zG += ( alpha * ( (G << zprec) - zG ) ) >> aprec;
    zB += ( alpha * ( (B << zprec) - zB ) ) >> aprec;
    zA += ( alpha * ( (A << zprec) - zA ) ) >> aprec;
    *bptr = zR >> zprec;
    *(bptr + 1) = zG >> zprec;
    *(bptr + 2) = zB >> zprec;
    *(bptr + 3) = zA >> zprec;
}

template<int aprec,int zprec>
static inline void blurrow( QImage & im, int line, int alpha)
{
    int zR,zG,zB,zA;
    QRgb *ptr = (QRgb *)im.scanLine(line);
    zR = *( (unsigned char *)ptr ) << zprec;
    zG = *( (unsigned char *)ptr + 1 ) << zprec;
    zB = *( (unsigned char *)ptr + 2 ) << zprec;
    zA = *( (unsigned char *)ptr + 3 ) << zprec;

    for(int index = 1; index<im.width(); index++)
        blurinner<aprec,zprec>( (unsigned char *)&ptr[index],zR,zG,zB,zA,alpha );

    for(int index = im.width() - 2; index>=0; index--)
        blurinner<aprec,zprec>( (unsigned char *)&ptr[index],zR,zG,zB,zA,alpha );
}

template<int aprec, int zprec>
static inline void blurcol( QImage & im, int col, int alpha)
{
    int zR,zG,zB,zA;
    QRgb *ptr = (QRgb *)im.bits();
    ptr += col;
    zR = *( (unsigned char *)ptr ) << zprec;
    zG = *( (unsigned char *)ptr + 1 ) << zprec;
    zB = *( (unsigned char *)ptr + 2 ) << zprec;
    zA = *( (unsigned char *)ptr + 3 ) << zprec;

    for( int index = im.width(); index<(im.height() - 1) * im.width(); index += im.width() )
        blurinner<aprec,zprec>( (unsigned char *)&ptr[index],zR,zG,zB,zA,alpha );

    for( int index = (im.height() - 2) * im.width(); index>=0; index -= im.width() )
        blurinner<aprec,zprec>( (unsigned char *)&ptr[index],zR,zG,zB,zA,alpha );
}

/*
 *  expblur(QImage &img, int radius)
 *
 *  In-place blur of image 'img' with kernel
 *  of approximate radius 'radius'.
 *
 *  Blurs with two sided exponential impulse
 *  response.
 *
 *  aprec = precision of alpha parameter
 *  in fixed-point format 0.aprec
 *
 *  zprec = precision of state parameters
 *  zR,zG,zB and zA in fp format 8.zprec
 */
template<int aprec,int zprec>
void expblur( QImage &img, int radius )
{
    if (radius < 1)
        return;

    // Calculate the alpha such that 90% of the kernel is within the radius. Kernel extends to infinity.
    int alpha = (int)( (1 << aprec) * ( 1.0f - expf( -2.3f / (radius + 1.f) ) ) );

    for(int row = 0; row<img.height(); row++)
        blurrow<aprec,zprec>(img,row,alpha);

    for(int col = 0; col<img.width(); col++)
        blurcol<aprec,zprec>(img,col,alpha);
}

BackgroundPainter::BackgroundPainter(QObject *parent)
    : QObject(parent),
    m_shareImageBetweenScreen(false),
    m_multiMonitorType(MULTIMONITOR_NO_MULTIMONITOR),
    m_resizingType(SCALE_KEEP_EXPANDED_ASPECT),
    m_verticalAlignmentType(ALIGN_V_CENTER),
    m_horizontalAlignmentType(ALIGN_H_CENTER),
    m_fillingType(FILL_BY_SMART_COLOR),
    m_specialEffectType(EFFECT_NONE),
    m_lastSpecialEffectType(EFFECT_NONE),
    m_specialEffectParam(5.0f),
    m_fillingColor1(Qt::red),
    m_fillingColor2(Qt::blue),
    m_inverseGradient(false),
    m_smoothAlgorithm(SMOOTH_DEFAULT),
    m_sharpenSmoothScale(false)
{
    loadSettings();

    // update
    m_hasParameter << EFFECT_BLUR;
    m_hasParameter << EFFECT_GAUSSIAN_BLUR;
    m_hasParameter << EFFECT_SHARPEN;
    m_hasParameter << EFFECT_GAUSSIAN_SHARPEN;
    m_hasParameter << EFFECT_EMBOSS;
    m_hasParameter << EFFECT_DESATURATE;
    m_hasParameter << EFFECT_SWIRL;
    m_hasParameter << EFFECT_OILPAINT;
    m_hasParameter << EFFECT_GLOW;
    m_hasParameter << EFFECT_PROGRESSIVEBLUR;
}

BackgroundPainter::~BackgroundPainter()
{
    //saveSettings();
}

QColor m_fillingColor;

void BackgroundPainter::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("BackgroundPainter");

    m_shareImageBetweenScreen = settings.value("shareImageBetweenScreen",false).toBool();

    m_multiMonitorType = settings.value("multiMonitorType",MULTIMONITOR_NO_MULTIMONITOR).toInt();
    m_resizingType = settings.value("resizingType",SCALE_KEEP_EXPANDED_ASPECT).toInt();
    m_verticalAlignmentType = settings.value("verticalAlignmentType",ALIGN_V_CENTER).toInt();
    m_horizontalAlignmentType = settings.value("horizontalAlignmentType",ALIGN_H_CENTER).toInt();
    m_fillingType = settings.value("fillingType",FILL_BY_SMART_COLOR).toInt();
    m_fillingColor1 = settings.value( "fillingColor1",QColor(5,150,200) ).value<QColor>();
    m_fillingColor2 = settings.value( "fillingColor2",QColor(5,250,200) ).value<QColor>();
    m_specialEffectType = settings.value("specialEffectType",EFFECT_NONE).toInt();
    m_specialEffectParam = settings.value("specialEffectParam",5.0f).toFloat();
    m_inverseGradient = settings.value("inverseGradient",false).toBool();
    m_smoothAlgorithm = settings.value("smoothAlgorithm",SMOOTH_DEFAULT).toInt();

    settings.endGroup();
}

void BackgroundPainter::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("BackgroundPainter");

    settings.setValue("shareImageBetweenScreen", m_shareImageBetweenScreen);
    settings.setValue("multiMonitorType", m_multiMonitorType);
    settings.setValue("resizingType", m_resizingType);
    if (m_verticalAlignmentType != ALIGN_V_USER)
        settings.setValue("verticalAlignmentType", m_verticalAlignmentType);
    if (m_verticalAlignmentType != ALIGN_H_USER)
        settings.setValue("horizontalAlignmentType", m_horizontalAlignmentType);
    settings.setValue("fillingType", m_fillingType);
    settings.setValue("fillingColor1", m_fillingColor1);
    settings.setValue("fillingColor2", m_fillingColor2);
    settings.setValue("specialEffectType", m_specialEffectType);
    settings.setValue("specialEffectParam", m_specialEffectParam);
    settings.setValue("inverseGradient", m_inverseGradient);
    settings.setValue("smoothAlgorithm", m_smoothAlgorithm);

    settings.endGroup();
}

void BackgroundPainter::setMultiMonitorSupportType(int val)
{
    m_multiMonitorType = val;
}

void BackgroundPainter::setResizingType(int method)
{
    m_resizingType = method;
}

void BackgroundPainter::setVerticalAlignmentType(int val)
{
    m_verticalAlignmentType = val;
}

void BackgroundPainter::setHorizontalAlignmentType(int val)
{
    m_horizontalAlignmentType = val;
}

void BackgroundPainter::setFillingType(int val)
{
    m_fillingType = val;
}

void BackgroundPainter::setSmoothAlgorithm(int val)
{
    m_smoothAlgorithm = val;
}

void BackgroundPainter::setSpecialEffectType(int val)
{
    m_specialEffectType = val;
}

void BackgroundPainter::setSpecialEffectParameter(float val)
{
    m_specialEffectParam = val;
}

void BackgroundPainter::setShareImageBetweenScreen(bool val)
{
    m_shareImageBetweenScreen = val;
}

void BackgroundPainter::setInverseGradient(bool val)
{
    m_inverseGradient = val;
}

void BackgroundPainter::setFillingColor1(const QColor &color)
{
    m_fillingColor1 = color;
}

void BackgroundPainter::setFillingColor2(const QColor &color)
{
    m_fillingColor2 = color;
}

QImage BackgroundPainter::processImage(const QImage &srcImage, bool previewMode)
{
    QSize resampledSrcImgSize = getSizeAccordingToConfig(srcImage.size(),previewMode);
    QSize resampledTargetImgSize = getSizeAccordingToConfig(m_targetSizeWithMargins,previewMode);

    QSize goodSize = resampledSrcImgSize.expandedTo(resampledTargetImgSize);

    QImage wallp(goodSize,QImage::Format_ARGB32);

    fillImage(srcImage,wallp);

    QImage img = smoothScale(srcImage,resampledSrcImgSize,m_smoothAlgorithm);

    postProcess(img, m_specialEffectType,previewMode);

    embedImage(img,wallp);

    return wallp;
}

QImage BackgroundPainter::smoothScale(const QImage &image,const QSize &newSize,int mode) const
{
    QImage img;
    float factor = 1.0f;
    if (m_sharpenSmoothScale)
        factor = 0.5f;
    switch(mode)
    {
        case SMOOTH_BLITZ_BLACKMANFILTER:
        {
            img = Blitz::smoothScaleFilter(image, newSize,
                                           factor,
                                           Blitz::BlackmanFilter,
                                           Qt::IgnoreAspectRatio);
            break;
        }
        case SMOOTH_BLITZ_CUBICFILTER:
        {
            img = Blitz::smoothScaleFilter(image, newSize,
                                           factor,
                                           Blitz::CubicFilter,
                                           Qt::IgnoreAspectRatio);
            break;
        }
        case SMOOTH_BLITZ_MITCHELLFILTER:
        {
            img = Blitz::smoothScaleFilter(image, newSize,
                                           factor,
                                           Blitz::MitchellFilter,
                                           Qt::IgnoreAspectRatio);
            break;
        }
        case SMOOTH_BLITZ_LANCZOSFILTER:
        {
            img = Blitz::smoothScaleFilter(image, newSize,
                                           factor,
                                           Blitz::LanczosFilter,
                                           Qt::IgnoreAspectRatio);
            break;
        }
        default:
        case SMOOTH_DEFAULT:
        {
            img = image.scaled( newSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
            break;
        }
    }

    return img;
}

void BackgroundPainter::embedImage(const QImage&src, QImage & dest)
{
    QPainter painter(&dest);

    int offsetDestX = MAX(0,( dest.width() - src.width() ) / 2);
    int offsetDestY = MAX(0,( dest.height() - src.height() ) / 2);
    int offsetSrcX = ABS( MIN(0,( dest.width() - src.width() ) / 2) );
    int offsetSrcY = ABS( MIN(0,( dest.height() - src.height() ) / 2) );
    int width = MIN( dest.width(),src.width() );
    int height = MIN( dest.height(),src.height() );

    QRect source (offsetSrcX, offsetSrcY,width, height);
    QRect target (offsetDestX, offsetDestY,width,height);
    painter.drawImage(target, src, source);
}

QColor BackgroundPainter::computeSmartColor(const QImage &img)
{
    int red = 0;
    int green = 0;
    int blue = 0;

    for (int y = 0; y < img.height(); y++)
    {
        const QRgb *lineSrc = reinterpret_cast<const QRgb *>( img.scanLine(y) );

        for (int x = 0; x < img.width(); x++)
        {
            red += qRed(lineSrc[x]);
            green += qGreen(lineSrc[x]);
            blue += qBlue(lineSrc[x]);
        }
    }

    //Calculate average
    red /= img.height() * img.width();
    green /= img.height() * img.width();
    blue /= img.height() * img.width();

    return QColor(red,green,blue);
}

void BackgroundPainter::postProcess(QImage &img, int effect, bool preview)
{
    float sizeFactor = 1.0f;

    if (preview)
        sizeFactor = previewFactor;

    switch (effect)
    {
        case EFFECT_RANDOM:
            if (!preview)
                m_lastSpecialEffectType = qrand() % EFFECT_RANDOM;
            qDebug() << "m_lastSpecialEffectType = " << m_lastSpecialEffectType;
            postProcess(img,m_lastSpecialEffectType,preview);
            break;

        case EFFECT_GRAYSCALE:
            Blitz::grayscale(img);
            break;

        case EFFECT_BLUR:
            img = Blitz::blur(img,(int) m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_GAUSSIAN_BLUR:
            img = Blitz::gaussianBlur(img,(int) m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_EDGE:
            img = Blitz::edge(img);
            break;

        case EFFECT_SHARPEN:
            img = Blitz::sharpen(img, (int) m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_GAUSSIAN_SHARPEN:
            img = Blitz::gaussianSharpen(img, (int) m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_EMBOSS:
            img = Blitz::emboss(img, (int) m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_DESATURATE:
            img = Blitz::desaturate(img, m_specialEffectParam);
            break;

        case EFFECT_FLATTEN:
            img = Blitz::flatten(img,  m_fillingColor1, m_fillingColor2);
            break;

        case EFFECT_SWIRL:
            img = Blitz::swirl(img, m_specialEffectParam);
            break;

        case EFFECT_OILPAINT:
            img = Blitz::oilPaint(img, m_specialEffectParam / sizeFactor);
            break;

        case EFFECT_CHARCOAL:
            img = Blitz::charcoal(img);
            break;

        case EFFECT_SEPIA:
            img = toSepia(img);
            break;

        case EFFECT_INVERTCOLOR:
            img = invertColors(img);
            break;

        case EFFECT_GLOW:
            img = glow(img,m_specialEffectParam);
            break;

        case EFFECT_PROGRESSIVEBLUR:
            img = progressiveBlur(img,m_specialEffectParam);
            break;

        case EFFECT_NONE:
        default:
            break;
    }
}

void BackgroundPainter::fillImage(const QImage &src, QImage &target)
{
    // fill the image with the default values if needed
    if (m_resizingType == SCALE_IGNORE_ASPECT ||
        m_resizingType == SCALE_KEEP_EXPANDED_ASPECT)
        return;

    QPainter painter(&target);

    switch (m_fillingType)
    {
        case FILL_BY_COLOR:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by color ...");
            painter.fillRect(target.rect(),m_fillingColor1);
            break;
        }

        case FILL_BY_SMART_COLOR:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by smart color ...");
            painter.fillRect( target.rect(),computeSmartColor(src) );
            break;
        }

        case FILL_BY_BLUR:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by blurred image ...");
            QImage blurred(src);
            blurred = Blitz::blur(blurred,30);

            //blurred.save("d:/tmp/blurred.png","PNG");

            painter.drawImage( target.rect(), blurred.scaled( target.width(),target.height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation) );
            break;
        }

        case FILL_BY_VERTICAL_GRADIENT:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by vertical gradient ...");
            QLinearGradient linearGrad( 0, 0, 0, target.height() );

            if (m_inverseGradient)
            {
                linearGrad.setColorAt( 0.0, m_fillingColor2);
                linearGrad.setColorAt( 1.0, m_fillingColor1);
            }
            else
            {
                linearGrad.setColorAt( 0.0, m_fillingColor1);
                linearGrad.setColorAt( 1.0, m_fillingColor2);
            }

            painter.setBrush( linearGrad );
            painter.fillRect( 0, 0, target.width(),target.height(), QBrush(linearGrad) );
            break;
        }

        case FILL_BY_HORIZONTAL_GRADIENT:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by horizontal gradient ...");
            QLinearGradient linearGrad( 0, 0, target.width(), 0 );

            if (m_inverseGradient)
            {
                linearGrad.setColorAt( 0.0, m_fillingColor2);
                linearGrad.setColorAt( 1.0, m_fillingColor1);
            }
            else
            {
                linearGrad.setColorAt( 0.0, m_fillingColor1);
                linearGrad.setColorAt( 1.0, m_fillingColor2);
            }

            painter.setBrush( linearGrad );
            painter.fillRect( 0, 0, target.width(),target.height(), QBrush(linearGrad) );
            break;
        }

        case FILL_BY_RADIAL_GRADIENT:
        {
            LogHandler::getInstance()->reportDebug("BackgroundPainter: Fill by radial gradient ...");
            float w = target.width() / 2.0f;
            float h = target.height() / 2.0f;
            QRadialGradient radialGrad( w,h, sqrtf(w * w + h * h) );

            if (m_inverseGradient)
            {
                radialGrad.setColorAt( 0.0, m_fillingColor2);
                radialGrad.setColorAt( 1.0, m_fillingColor1);
            }
            else
            {
                radialGrad.setColorAt( 0.0, m_fillingColor1);
                radialGrad.setColorAt( 1.0, m_fillingColor2);
            }

            painter.setBrush( radialGrad );
            painter.fillRect( 0, 0, target.width(),target.height(), QBrush(radialGrad) );
            break;
        }

        default:
        {
            break;
        }
    }
}

QSize BackgroundPainter::getSizeAccordingToConfig(const QSize &baseSize, bool preview)
{
    QSize finalSize(baseSize);

    // nothing to do !!
    if (baseSize == m_targetSizeWithMargins)
    {
        if (preview)
            finalSize /= previewFactor;

        return finalSize;
    }

    float baseAspectRatio = (float) baseSize.width() / (float) ( baseSize.height() );
    float targetAspectRatio = (float) m_targetSizeWithMargins.width() / (float) ( m_targetSizeWithMargins.height() );

    switch (m_resizingType)
    {
        case SCALE_NO_RESIZE:
        {
            finalSize = baseSize;
            break;
        }

        case SCALE_IGNORE_ASPECT:
        {
            finalSize = m_targetSizeWithMargins;
            break;
        }

        case SCALE_KEEP_ASPECT:
        {
            if ( (baseAspectRatio >= targetAspectRatio) )
                finalSize = QSize( m_targetSizeWithMargins.width(), (int) (m_targetSizeWithMargins.width() / baseAspectRatio + 0.5) );
            else
                finalSize = QSize( (int) (m_targetSizeWithMargins.height() * baseAspectRatio + 0.5), m_targetSizeWithMargins.height() );

            break;
        }

        case SCALE_KEEP_EXPANDED_ASPECT:
        {
            if (baseAspectRatio >= targetAspectRatio )
                finalSize = QSize( (int)(baseSize.width() / ( (float)baseSize.height() / m_targetSizeWithMargins.height() ) + 0.5),m_targetSizeWithMargins.height() );
            else
                finalSize = QSize( m_targetSizeWithMargins.width(), (int)(baseSize.height() / ( (float)baseSize.width() / m_targetSizeWithMargins.width() ) + 0.5) );

            break;
        }
    }

    if (preview)
        finalSize /= previewFactor;

    return finalSize;
}

QList<QImage> BackgroundPainter::cropAndSplit(const QImage & background,const QList<QRectF> &listGamingArea)
{
    QSize finalSize(m_targetSize);
    float sizeFactor = 1.0f;
    QList<QImage> results;

    finalSize /= sizeFactor;

    QRectF total;
    QSize bgSize = background.size();

    foreach (const QRectF &rectf, listGamingArea)
    total |= rectf;

    int marginsWidth = bgSize.width() - total.width() - total.x();

    int alignOffsetX = 0;
    int alignOffsetY = 0;

    if ( !m_userOffset.isNull() )
    {
        alignOffsetY = m_userOffset.y() / sizeFactor;
        alignOffsetX = m_userOffset.x() / sizeFactor;
    }
    else
    {
        // compute the offset according to the alignment requirement
        switch(m_verticalAlignmentType)
        {
            case ALIGN_V_CENTER:
                alignOffsetY = ( bgSize.height() - finalSize.height() ) / 2;
                break;

            case ALIGN_V_BOTTOM:
                alignOffsetY = bgSize.height() - finalSize.height();
                break;

            case ALIGN_V_TOP: // top align by default
            default:
                break;
        }

        switch(m_horizontalAlignmentType)
        {
            case ALIGN_H_CENTER:
                alignOffsetX = ( bgSize.width() - finalSize.width() ) / 2;
                break;

            case ALIGN_H_RIGHT:
                alignOffsetX = marginsWidth;
                break;

            case ALIGN_H_LEFT: // top align by default
            default:
                break;
        }
    }

    // Fill the image with the gaming area list
    foreach (const QRectF &zone, listGamingArea)
    {
        QRectF source(
            zone.x() / sizeFactor + alignOffsetX,
            zone.y() / sizeFactor + alignOffsetY,
            zone.width() / sizeFactor,
            zone.height() / sizeFactor
            );

        results << background.copy( source.toRect() );
    }
    return results;
}

QImage BackgroundPainter::cropAndMerge(const QImage & background,const QList<QRectF> &listGamingArea)
{
    QSize finalSize(m_targetSize);
    float sizeFactor = 1.0f;

    finalSize /= sizeFactor;

    QRectF total;
    QSize bgSize = background.size();

    foreach (const QRectF &rectf, listGamingArea)
    total |= rectf;

    int marginsWidth = bgSize.width() - total.width() - total.x();

    QImage wallp(finalSize,QImage::Format_ARGB32);

    int alignOffsetX = 0;
    int alignOffsetY = 0;

    if ( !m_userOffset.isNull() )
    {
        alignOffsetY = m_userOffset.y() / sizeFactor;
        alignOffsetX = m_userOffset.x() / sizeFactor;
    }
    else
    {
        // compute the offset according to the alignment requirement
        switch(m_verticalAlignmentType)
        {
            case ALIGN_V_CENTER:
                alignOffsetY = ( bgSize.height() - finalSize.height() ) / 2;
                break;

            case ALIGN_V_BOTTOM:
                alignOffsetY = bgSize.height() - finalSize.height();
                break;

            case ALIGN_V_TOP: // top align by default
            default:
                break;
        }

        switch(m_horizontalAlignmentType)
        {
            case ALIGN_H_CENTER:
                alignOffsetX = qMin(marginsWidth,( bgSize.width() - finalSize.width() ) / 2);
                break;

            case ALIGN_H_RIGHT:
                alignOffsetX = marginsWidth; //bgSize.width() - finalSize.width();
                break;

            case ALIGN_H_LEFT: // left align by default
            default:
                break;
        }
    }

    QPainter painter(&wallp);
    int offsetX = MAX(0,( finalSize.width() - bgSize.width() ) / 2);
    int offsetY = 0;

    // Fill the image with the gaming area list
    foreach (const QRectF &zone, listGamingArea)
    {
        QRectF target(offsetX,offsetY + zone.y(),zone.width() / sizeFactor,zone.height() / sizeFactor);
        QRectF source(
            zone.x() / sizeFactor + alignOffsetX,
            zone.y() / sizeFactor + alignOffsetY,
            zone.width() / sizeFactor,
            zone.height() / sizeFactor
            );

        painter.drawImage(target,background,source);
        offsetX += zone.width() / sizeFactor;
    }
    return wallp;
}

QImage BackgroundPainter::crop(const QImage & background, const QSize &size, bool preview)
{
    QSize finalSize(size);
    float sizeFactor = 1.0f;

    if (preview)
        sizeFactor = previewFactor;

    finalSize /= sizeFactor;

    QImage wallp(finalSize,QImage::Format_ARGB32);
    QSize bgSize = background.size();
    int marginsWidth = bgSize.width() - finalSize.width();

    int alignOffsetX = 0;
    int alignOffsetY = 0;

    if ( !m_userOffset.isNull() )
    {
        alignOffsetY = m_userOffset.y() / sizeFactor;
        alignOffsetX = m_userOffset.x() / sizeFactor;
    }
    else
    {
        // compute the offset according to the alignment requirement
        switch(m_verticalAlignmentType)
        {
            case ALIGN_V_CENTER:
                alignOffsetY = ( bgSize.height() - finalSize.height() ) / 2;
                break;

            case ALIGN_V_BOTTOM:
                alignOffsetY = bgSize.height() - finalSize.height();
                break;

            case ALIGN_V_TOP: // top align by default
            default:
                break;
        }

        switch(m_horizontalAlignmentType)
        {
            case ALIGN_H_CENTER:
                alignOffsetX = ( bgSize.width() - finalSize.width() ) / 2;
                break;

            case ALIGN_H_RIGHT:
                alignOffsetX = marginsWidth; //bgSize.width() - finalSize.width();
                break;

            case ALIGN_H_LEFT: // left align by default
            default:
                break;
        }
    }

    QPainter painter(&wallp);

    QRectF target( 0.0f,0.0f,finalSize.width(),finalSize.height() );
    QRectF source(
        alignOffsetX,
        alignOffsetY,
        finalSize.width(),
        finalSize.height()
        );

    painter.drawImage(target,background,source);

    return wallp;
}

bool BackgroundPainter::hasSpecialEffectParameter(int specialEffect) const
{
    if ( m_hasParameter.contains(specialEffect) )
        return true;

    return false;
}

QImage BackgroundPainter::toSepia(const QImage &img)
{
    QImage dest(img.size(), QImage::Format_ARGB32);
    const int w = img.width();
    const int h = img.height();

    for (int y = 0; y<h; y++)
        for (int x = 0; x<w; x++)
        {
            QRgb pixel = img.pixel(x, y);
            const unsigned int average = ( qGreen(pixel) + qRed(pixel) + qBlue(pixel) ) / 3;
            const int red = average * 1.176, green = average * 0.837, blue = average * 0.558;
            pixel = qRgba( (red <= 255) ? red : 255, (green <= 255) ? green : 255, (blue <= 255) ? blue : 255, qAlpha(pixel) );
            dest.setPixel(x,y,pixel);
        }

    return dest;
}

QImage BackgroundPainter::progressiveBlur(const QImage &image, int radius)
{
    QImage blurredImage = image.convertToFormat(QImage::Format_ARGB32); // be careful for playing with alpha and scanLine
    //use Format_ARGB32 instead of Format_ARGB32_Premultiplied

    // create radial alpha mask
    QImage mask(image.size(),QImage::Format_ARGB32);
    QPainter painterMask(&mask);
    float w = mask.width() / 2.0f;
    float h = mask.height() / 2.0f;
    QRadialGradient radialGrad( w,h, qMin(w,h) );
    radialGrad.setColorAt( 0.2, QColor(Qt::black) );
    radialGrad.setColorAt( 1.0, QColor(Qt::white) );
    painterMask.setBrush( radialGrad );
    painterMask.fillRect( 0, 0, mask.width(),mask.height(), QBrush(radialGrad) );
    painterMask.end();

    //ExpBlur with 0.16 fp for alpha and
    //8.7 fp for state parameters zR,zG,zB and zA
    QImage blurred = image.convertToFormat(QImage::Format_ARGB32);
    expblur<16,7>(blurred, radius);

    // replace alpha value in blurred image
    int width = blurred.width();
    int height = blurred.height();

    for (int y = 0; y < height; y++)
    {
        QRgb *lineSrc = reinterpret_cast<QRgb *>( blurredImage.scanLine(y) );
        QRgb *lineBlur = reinterpret_cast<QRgb *>( blurred.scanLine(y) );
        QRgb *lineMask = reinterpret_cast<QRgb *>( mask.scanLine(y) );

        for (int x = 0; x < width; x++)
        {
            QRgb valueDest = qRgba( qRed(*lineBlur),qGreen(*lineBlur),qBlue(*lineBlur),qRed(*lineMask) );

            blurred.setPixel(x,y,valueDest);

            lineMask++;
            lineSrc++;
            lineBlur++;
        }
    }

    QPainter imagePainter(&blurredImage);

    imagePainter.save();
    imagePainter.setCompositionMode(QPainter::CompositionMode_SourceOver);
    imagePainter.drawImage(0,0, blurred);
    imagePainter.restore();
    imagePainter.end();

    return blurredImage;
}

QImage BackgroundPainter::glow(const QImage &image, int radius)
{
    QImage glownImage(image.size(), QImage::Format_ARGB32_Premultiplied);
    glownImage.fill(0x00);
    QPainter imagePainter(&glownImage);

    imagePainter.drawImage(0,0, image);

    //ExpBlur with 0.16 fp for alpha and
    //8.7 fp for state parameters zR,zG,zB and zA
    QImage blurred = image;
    expblur<16,7>(blurred, radius);

    imagePainter.save();
    imagePainter.setCompositionMode(QPainter::CompositionMode_Plus);
    imagePainter.drawImage(0,0, blurred);
    imagePainter.restore();
    imagePainter.end();

    return glownImage;
}

QImage BackgroundPainter::invertColors(const QImage &img)
{
    QImage inverted(img.size(), QImage::Format_ARGB32);
    const int w = img.width();
    const int h = img.height();

    for (int y = 0; y<h; y++)
        for (int x = 0; x<w; x++)
        {
            QRgb pixel = img.pixel(x, y);
            pixel = qRgba( 255 - qRed(pixel), 255 - qGreen(pixel), 255 - qBlue(pixel), qAlpha(pixel) );
            inverted.setPixel(x,y,pixel);
        }

    return inverted;
}

QImage BackgroundPainter::transform(const QImage &image, int mode) const
{
    QImage result = image;

    switch (mode)
    {
        case ROTATION_NONE:
            break;

        case ROTATION_FLIP_HORIZ:

            // image.transformed(QTransform().rotate(180, Qt::YAxis));
            result = result.mirrored(true,false);
            break;

        case ROTATION_180:

            // image.transformed(QTransform().rotate(180, Qt::ZAxis));
            result = result.mirrored(true,true);
            break;

        case ROTATION_FLIP_VERTICAL:

            // image.transformed(QTransform().rotate(180, Qt::XAxis));
            result = result.mirrored(false,true);
            break;

        case ROTATION_TRANSPOSE:

            // image.transformed(QTransform().rotate(180, Qt::YAxis).rotate(90, Qt::ZAxis));
            result = result.transformed(QTransform().rotate(90),Qt::SmoothTransformation).mirrored(true,false);
            break;

        case ROTATION_90:

            // image.transformed(QTransform().rotate(90, Qt::ZAxis));
            result = result.transformed(QTransform().rotate(90),Qt::SmoothTransformation);
            break;

        case ROTATION_TRANSVERSE:

            // image.transformed(QTransform().rotate(180, Qt::XAxis).rotate(90, Qt::ZAxis));
            result = result.transformed(QTransform().rotate(270),Qt::SmoothTransformation).mirrored(true,false);
            break;

        case ROTATION_270:

            // image.transformed(QTransform().rotate(270, Qt::ZAxis));
            result = result.transformed(QTransform().rotate(270),Qt::SmoothTransformation);
            break;

        case ROTATION_UNLNOWN:
        default:
            break;
    }

    return result;
}

void BackgroundPainter::setUserOffset(const QPointF &pt)
{
    m_userOffset = pt;
}
