
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include "DesktopScene.h"
#include "ScreenItem.h"
#include "BackgroundItem.h"

#include <QtCore/QDebug>
#include <QtGui/QApplication>
#include <QtGui/QPalette>

const int displayedMargin = 2;

bool lessThanCenter(const QGraphicsItem *s1, const QGraphicsItem *s2)
{
    QPointF p1 = s1->pos();
    QPointF p2 = s2->pos();
    return p1.x() < p2.x();
}

DesktopScene::DesktopScene(QObject * parent)
    : QGraphicsScene(parent),
    m_hasMove(false),
    m_state(STATE_IDLE)
{
    setSceneRect(0,0,200, 200);

    // add a background item by default
    BackgroundItem *item = new BackgroundItem;
    addItem(item);
}

DesktopScene::~DesktopScene()
{
}

void DesktopScene::addScreens(const QList<ScreenItem *> &screens)
{
    int offset = 0;
    foreach (ScreenItem * screen, screens)
    {
        addItem(screen);

        // decal for auto placement
        if (screen->x() == 0)
            screen->setX(offset);
        offset += screen->boundingRect().width();
    }
    reorder();
}

void DesktopScene::addScreen(ScreenItem *screen)
{
    addItem(screen);

    // decal for auto placement
    screen->setX( screen->boundingRect().width() );
    reorder();
}

void DesktopScene::reorder()
{
    // search for cumulated width & height
    QList<QGraphicsItem *> list = items ();

    // sort list with x value => get the correct order for screen to be displayed
    qSort(list.begin(),list.end(),lessThanCenter);

    // Get the higher screen for vertical alignment
    int highest = 0;
    foreach( QGraphicsItem * item, list )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
            highest = qMax( highest,screen->getSizeWithMargins ().height() );
    }

    // place all item now
    int offsetX = 0;
    int offsetY = 0;

    foreach( QGraphicsItem * item, list )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (!screen)
            continue;

        QSize size = screen->getSizeWithMargins ();
        QPointF oldScenePos = screen->scenePos();
        QPointF oldPos = screen->pos();
        QRectF oldSceneRect = screen->sceneBoundingRect();

        int posx = offsetX;
        posx += size.width() / 2; // because center on (0,0)
        int posy = offsetY;
        posy += size.height() / 2; // because center on (0,0)
        posy -= screen->getMarginBottom();

        if (oldPos.y() > highest / 2) // bottom alignment
        {
            posy += ( highest - size.height() );
        }

        screen->setPos(posx,posy);

        offsetX += screen->getSizeWithMargins().width(); // + displayedMargin;
    }

    emit reordering();
}

void DesktopScene::applyImage(const QImage &cropped,const QImage &full, const QSize &desktopSize, const QSize &realSize)
{
    foreach( QGraphicsItem * item, items () )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            screen->setImage(cropped);
        }
        else
        {
            BackgroundItem *bg = dynamic_cast<BackgroundItem*>(item);
            if (bg)
                bg->setImage(full,realSize,desktopSize);
        }
    }
}

void DesktopScene::setUserOffset(const QPoint &offset)
{
    foreach( QGraphicsItem * item, items () )
    {
        BackgroundItem *bg = dynamic_cast<BackgroundItem*>(item);
        if (bg)
            bg->setOffset(offset);
    }
}

QPointF DesktopScene::getUserOffset() const
{
    QPointF offset(0,0);

    foreach( QGraphicsItem * item, items () )
    {
        BackgroundItem *bg = dynamic_cast<BackgroundItem*>(item);
        if ( bg && bg->isVisible() )
            offset = bg->getOffset();
    }
    return offset;
}

QList<QRectF>  DesktopScene::getGamingAreaList() const
{
    QList<QRectF> gamingArea;

    QList<QGraphicsItem *> list = items ();

    // search if background item is visible => get user offset

    // sort list with x value => get the correct order for screen to be displayed
    qSort(list.begin(),list.end(),lessThanCenter);

    foreach( QGraphicsItem * item, list )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            gamingArea << screen->getValidSceneBoundingRect();
        }
    }
    return gamingArea;
}

QSize DesktopScene::getSize() const
{
    QSize size(0,0);
    foreach( QGraphicsItem * item, items() )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            size.rwidth() += screen->getSize().width();
            size.rheight() = qMax( size.height(), screen->getSize().height() );
        }
    }
    return size;
}

QSize DesktopScene::getSizeWithMargins() const
{
    QSize size(0,0);
    foreach( QGraphicsItem * item, items() )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            size.rwidth() += screen->getRealSizeWithMargins().width();
            size.rheight() = qMax( size.height(), screen->getRealSizeWithMargins().height() );
        }
    }
    return size;
}

void DesktopScene::mouseMoveEvent ( QGraphicsSceneMouseEvent * mouseEvent )
{
    m_hasMove = true;
    QGraphicsScene::mouseMoveEvent(mouseEvent);
}

void DesktopScene::mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent )
{
    if (m_hasMove && m_state == STATE_MOVING_SCREEN)
        reorder();
    QGraphicsScene::mouseReleaseEvent(mouseEvent);
    m_hasMove = false;
}

void DesktopScene::setSceneState(int state)
{
    m_state = state;

    foreach( QGraphicsItem * item, items () )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            screen->setFlag(QGraphicsItem::ItemIsMovable,m_state == STATE_MOVING_SCREEN);
            screen->setFlag(QGraphicsItem::ItemIsSelectable,m_state == STATE_MOVING_SCREEN);
            screen->setFlag(QGraphicsItem::ItemIsFocusable,m_state == STATE_MOVING_SCREEN);
            screen->setOpaque(m_state != STATE_MOVING_BACKGROUND);
        }
        else
        {
            BackgroundItem *bg = dynamic_cast<BackgroundItem*>(item);
            if (bg)
            {
                bg->setVisible(m_state == STATE_MOVING_BACKGROUND);
                bg->setFlag(QGraphicsItem::ItemIsMovable,m_state == STATE_MOVING_BACKGROUND);
                bg->setFlag(QGraphicsItem::ItemIsSelectable,m_state == STATE_MOVING_BACKGROUND);
                bg->setFlag(QGraphicsItem::ItemIsFocusable,m_state == STATE_MOVING_BACKGROUND);
            }
        }
    }
}

int DesktopScene::getSceneState() const
{
    return m_state;
}

QRectF DesktopScene::getSceneRect () const
{
    if (m_state == STATE_MOVING_BACKGROUND)
    {
        foreach( QGraphicsItem * item, items () )
        {
            BackgroundItem *bg = dynamic_cast<BackgroundItem*>(item);
            if (bg)
            {
                // compute the enlarge scene bounding box
                QRectF bgr = bg->boundingRect();
                QRectF sum( getScreensRect () );
                qreal diffWidth = bgr.width() - sum.width();
                qreal diffHeight = bgr.height() - sum.height();
                sum.setLeft(sum.left() - diffWidth);
                sum.setTop(sum.top() - diffHeight);
                sum.setBottom(sum.bottom() + diffHeight);
                sum.setRight(sum.right() + diffWidth);

                return sum;
            }
        }
        return QRectF ();
    }
    else
    {
        return getScreensRect ();
    }
}

QRectF DesktopScene::getScreensRect () const
{
    QRectF sum;
    foreach( QGraphicsItem * item, items () )
    {
        ScreenItem *screen = dynamic_cast<ScreenItem*>(item);
        if (screen)
        {
            QRectF rect = screen->sceneBoundingRect();
            sum |= rect;
        }
    }
    return sum;
}
