
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QtDebug>
#include <QtGui/QResizeEvent>
#include <QtGui/QGridLayout>
#include <QtGui/QLabel>

#include "ImageView.h"

//-----------------------------------------------------------------------------------------
// Image View

ImageView::ImageView(QWidget *parent, Qt::WFlags flags) : QGraphicsView(parent),
    m_displayInfos(false),
    m_navigatorWindow(NULL),
    m_navigatorSize(0.15f),
    m_navigatorMargin(10),
    m_showNavigator(true)

{
    m_viewportRect = QRect( 0, 0, width(), height() );
    m_worldMatrix.reset();

    m_navigatorWindow = new ImageNavigator(this, flags);
    m_navigatorWindow->setTransforms(&m_worldMatrix, &m_imgMatrix);

    initBackground();

    setStyleSheet( "QGraphicsView { border-style: none; }" );
    setMouseTracking (true); //receive mouse event everytime

    connect( m_navigatorWindow, SIGNAL( moveViewSignal(const QPointF &) ), this, SLOT( moveView(const QPointF &) ) );
}

ImageView::~ImageView()
{
}

void ImageView::initBackground()
{
    // precreate background tile
    m_tileBg = QPixmap(16, 16);
    m_tileBg.fill(Qt::white);
    QPainter pt(&m_tileBg);
    QColor color(202, 202, 202);
    pt.fillRect(0, 0, 8, 8, color);
    pt.fillRect(8, 8, 8, 8, color);
    pt.end();
}

void ImageView::setImage(const QImage &newImg)
{
    m_navigatorWindow->setImage( QImage() );  // clear overview

    m_image = newImg;
    m_imgRect = QRectF( 0, 0, newImg.width(), newImg.height() );

    m_worldMatrix.reset();
    updateImageMatrix();

    m_navigatorWindow->setImage(m_image);

    update();
}

bool ImageView::isImageInside()
{
    return m_viewportRect.contains( m_worldMatrix.mapRect(m_imgViewRect) );
}

// displacement functions --------------------------------------------------------------------

// Move Left
void ImageView::moveLeft()
{
    float delta = 2 * width() / ( 100.0 * m_worldMatrix.m11() );
    moveView( QPointF(delta,0) );
}

// Move Right
void ImageView::moveRight()
{
    float delta = -2 * width() / ( 100.0 * m_worldMatrix.m11() );
    moveView( QPointF(delta,0) );
}

// Move Up
void ImageView::moveUp()
{
    float delta = 2 * height() / ( 100.0 * m_worldMatrix.m11() );
    moveView( QPointF(0,delta) );
}

// Move Down
void ImageView::moveDown()
{
    float delta = -2 * height() / ( 100.0 * m_worldMatrix.m11() );
    moveView( QPointF(0,delta) );
}

void ImageView::setShowOverview(bool val)
{
    m_showNavigator = val;
    update();
}

void ImageView::zoomIn()
{
    zoom(1.1f);
}

void ImageView::zoomOut()
{
    zoom(0.9f);
}

void ImageView::zoom(float factor, const QPointF &c)
{
    if ( m_image.isNull() )
        return;

    QPointF center(c);

    //limit zoom out ---
    if (m_worldMatrix.m11() == 1 && factor < 1)
        return;

    if (m_worldMatrix.m11() * factor < 1)
    {
        resetView();
        return;
    }

    //limit zoom in ---
    if (m_worldMatrix.m11() * m_imgMatrix.m11() > 50 && factor > 1)
        return;

    bool blackBorder = false;

    // if no center assigned: zoom in at the image center
    if (center.x() == -1 || center.y() == -1)
    {
        center = QPointF( (float)width() / 2.0f, (float)height() / 2.0f );
    }
    else
    {
        // if black border - do not zoom to the mouse coordinate
        if ( (float)m_imgViewRect.width() * (m_worldMatrix.m11() * factor) < (float)width() )
        {
            center.setX( (float)width() / 2.0f );
            blackBorder = true;
        }

        if ( ( (float)m_imgViewRect.height() * m_worldMatrix.m11() * factor ) < (float)height() )
        {
            center.setY( (float)height() / 2.0f );
            blackBorder = true;
        }
    }

    //inverse the transform
    int a, b;
    m_worldMatrix.inverted().map(center.x(), center.y(), &a, &b);

    m_worldMatrix.translate(a - factor * a, b - factor * b);
    m_worldMatrix.scale(factor, factor);

    controlImagePosition();

    if (blackBorder && factor < 1)
        centerImage();

    emit newZoomFactor( getZoom() );

    update();
}

void ImageView::resetView()
{
    m_worldMatrix.reset();
    update();
}

float ImageView::getZoom()
{
    return m_imgMatrix.m11() * m_worldMatrix.m11() * 100.0f;
}

void ImageView::moveView(const QPointF &d)
{
    QPointF delta(d);
    QRectF imgWorldRect = m_worldMatrix.mapRect(m_imgViewRect);

    if ( imgWorldRect.width() < width() )
        delta.setX(0);

    if ( imgWorldRect.height() < height() )
        delta.setY(0);

    m_worldMatrix.translate( delta.x(), delta.y() );
    controlImagePosition();
    update();
}

void ImageView::controlImagePosition(float lb, float ub)
{
    QRectF imgRectWorld = m_worldMatrix.mapRect(m_imgViewRect);

    if (lb == -1)
        lb = m_viewportRect.width() / 2;

    if (ub == -1)
        ub = m_viewportRect.height() / 2;

    if ( imgRectWorld.left() > lb && imgRectWorld.width() > width() )
        m_worldMatrix.translate( ( lb - imgRectWorld.left() ) / m_worldMatrix.m11(), 0 );

    if ( imgRectWorld.top() > ub && imgRectWorld.height() > height() )
        m_worldMatrix.translate( 0, ( ub - imgRectWorld.top() ) / m_worldMatrix.m11() );

    if ( imgRectWorld.right() < lb && imgRectWorld.width() > width() )
        m_worldMatrix.translate( ( lb - imgRectWorld.right() ) / m_worldMatrix.m11(), 0 );

    if ( imgRectWorld.bottom() < ub && imgRectWorld.height() > height() )
        m_worldMatrix.translate( 0, ( ub - imgRectWorld.bottom() ) / m_worldMatrix.m11() );
}

void ImageView::centerImage()
{
    QRectF imgWorldRect = m_worldMatrix.mapRect(m_imgViewRect);

    // if black border - center the image
    if ( imgWorldRect.width() < (float)width() )
    {
        float dx = ( (float)width() - imgWorldRect.width() ) * 0.5f - m_imgViewRect.x() * m_worldMatrix.m11();
        dx = ( dx - m_worldMatrix.dx() ) / m_worldMatrix.m11();
        m_worldMatrix.translate(dx, 0);
    }
    else if (imgWorldRect.left() > 0)
        m_worldMatrix.translate(-imgWorldRect.left() / m_worldMatrix.m11(), 0);
    else if ( imgWorldRect.right() < width() )
        m_worldMatrix.translate( ( width() - imgWorldRect.right() ) / m_worldMatrix.m11(), 0 );

    if ( imgWorldRect.height() < height() )
    {
        float dy = ( height() - imgWorldRect.height() ) * 0.5f - m_imgViewRect.y() * m_worldMatrix.m22();
        dy = ( dy - m_worldMatrix.dy() ) / m_worldMatrix.m22();
        m_worldMatrix.translate(0, dy);
    }
    else if (imgWorldRect.top() > 0)
    {
        m_worldMatrix.translate( 0, -imgWorldRect.top() / m_worldMatrix.m22() );
    }
    else if ( imgWorldRect.bottom() < height() )
    {
        m_worldMatrix.translate( 0, ( height() - imgWorldRect.bottom() ) / m_worldMatrix.m22() );
    }
}

void ImageView::updateImageMatrix()
{
    if ( m_image.isNull() )
        return;

    QRectF oldImgRect = m_imgViewRect;
    QTransform oldImgMatrix = m_imgMatrix;

    m_imgMatrix.reset();

    // if the image is smaller or zoom is active: paint the image as is
    if ( !m_viewportRect.contains(m_imgRect) )
    {
        m_imgMatrix = getScaledImageMatrix();
    }
    else
    {
        m_imgMatrix.translate( (float)( width() - m_image.width() ) * 0.5f, (float)( height() - m_image.height() ) * 0.5f );
        m_imgMatrix.scale(1.0f, 1.0f);
    }

    m_imgViewRect = m_imgMatrix.mapRect(m_imgRect);

    // update world matrix
    if (m_worldMatrix.m11() != 1)
    {
        float scaleFactor = oldImgMatrix.m11() / m_imgMatrix.m11();
        double dx = oldImgRect.x() / scaleFactor - m_imgViewRect.x();
        double dy = oldImgRect.y() / scaleFactor - m_imgViewRect.y();

        m_worldMatrix.scale(scaleFactor, scaleFactor);
        m_worldMatrix.translate(dx, dy);
    }
}

QTransform ImageView::getScaledImageMatrix()
{
    // the image resizes as we zoom
    float ratioImg = m_imgRect.width() / m_imgRect.height();
    float ratioWin = (float)width() / (float)height();

    QTransform imgMatrix;
    float s;

    if (m_imgRect.width() == 0 || m_imgRect.height() == 0)
        s = 1.0f;
    else
        s = (ratioImg > ratioWin) ? (float)width() / m_imgRect.width() : (float)height() / m_imgRect.height();

    imgMatrix.scale(s, s);

    QRectF imgViewRect = imgMatrix.mapRect(m_imgRect);
    imgMatrix.translate( ( width() - imgViewRect.width() ) * 0.5f / s, ( height() - imgViewRect.height() ) * 0.5f / s );

    return imgMatrix;
}

void ImageView::paintEvent(QPaintEvent* event)
{
    QPainter painter( viewport() );

    // draw background
    painter.drawTiledPixmap(rect(), m_tileBg);

    if ( !m_image.isNull() )
    {
        painter.setWorldTransform(m_worldMatrix);

        if (m_imgMatrix.m11() * m_worldMatrix.m11() <= (float)2.0f)
            painter.setRenderHint(QPainter::SmoothPixmapTransform);

        draw(&painter);

        //Now disable matrixWorld for overlay display
        painter.setWorldMatrixEnabled (false);
    }

    //in mode zoom/panning
    if (m_worldMatrix.m11() > 1 && !isImageInside() && m_showNavigator)
    {
        if ( m_navigatorWindow->isVisible() )
        {
            // top right placement
            m_navigatorWindow->move(width() - m_navigatorMargin - m_navigatorWindow->width(), m_navigatorMargin);
        }
        else
        {
            m_navigatorWindow->show();
            m_navigatorWindow->update();
        }
    }
    else
    {
        m_navigatorWindow->hide();
    }

    QGraphicsView::paintEvent(event);
}

// drawing functions --------------------------------------------------------------------

void ImageView::draw(QPainter *painter)
{
    painter->setRenderHint(QPainter::SmoothPixmapTransform);    //-> uncomment for smooth aliasing
    painter->drawImage(m_imgViewRect, m_image, m_imgRect);
}

// event --------------------------------------------------------------------

void ImageView::resizeEvent(QResizeEvent *event)
{
    m_viewportRect = QRect( 0, 0, width(), height() );

    updateImageMatrix();
    centerImage();

    emit newZoomFactor( getZoom() );

    m_navigatorWindow->resize(size() * m_navigatorSize);
    m_navigatorWindow->setViewPortRect( geometry() );

    QGraphicsView::resizeEvent(event);
}

// key events --------------------------------------------------------------------

void ImageView::keyPressEvent(QKeyEvent* event)
{
    if (event->key() == Qt::Key_Up)
    {
        moveUp();
    }
    else if (event->key() == Qt::Key_Down)
    {
        moveDown();
    }
    else if (event->key() == Qt::Key_Left)
    {
        moveLeft();
    }
    else if (event->key() == Qt::Key_Right)
    {
        moveRight();
    }
    else if (event->key() == Qt::Key_Plus)
    {
        zoomIn();
    }
    else if (event->key() == Qt::Key_Minus)
    {
        zoomOut();
    }
    QGraphicsView::keyPressEvent(event);
}

// mouse events --------------------------------------------------------------------

void ImageView::mousePressEvent(QMouseEvent *event)
{
    m_enterPos = event->pos();
    setCursor(Qt::ClosedHandCursor);

    if (m_worldMatrix.m11() > 1 && !isImageInside() && event->buttons() == Qt::LeftButton)
        m_lastPos = event->pos();

    QWidget::mousePressEvent(event);
}

void ImageView::mouseReleaseEvent(QMouseEvent *event)
{
    setCursor(Qt::ArrowCursor);
    QWidget::mouseReleaseEvent(event);
}

void ImageView::mouseMoveEvent(QMouseEvent *event)
{
    if (m_displayInfos)
        getPixelInfo( event->pos() );

    if (m_worldMatrix.m11() > 1 && event->buttons() == Qt::LeftButton)
    {
        QPointF cPos = event->pos();
        QPointF dxy = (cPos - m_lastPos);
        m_lastPos = cPos;
        moveView( dxy / m_worldMatrix.m11() );
    }

    QWidget::mouseMoveEvent(event);
}

void ImageView::wheelEvent(QWheelEvent *event)
{
    float factor = event->delta();

    factor /= -1200.0f;
    factor += 1.0f;

    zoom( factor, event->pos() );
}

void ImageView::getPixelInfo(const QPoint& pos)
{
    if ( m_image.isNull() )
        return;

    QPointF imgPos = m_worldMatrix.inverted().map( QPointF(pos) );
    imgPos = m_imgMatrix.inverted().map(imgPos);

    QPoint xy = imgPos.toPoint();

    if ( xy.x() < 0 || xy.y() < 0 || xy.x() >= m_image.width() || xy.y() >= m_image.height() )
        return;

    QColor col = m_image.pixel(xy);

    QString msg = QString( tr("x: %1 y: %2 | r: %3 g: %4 b: %5") ).arg( QString::number( xy.x() ) ).arg( QString::number( xy.y() ) ).arg( QString::number( col.red() ) ).arg( QString::number( col.green() ) ).arg( QString::number( col.blue() ) );

    if ( m_image.hasAlphaChannel() )
        msg = msg + QString( tr(" a: %1") ).arg( QString::number( col.alpha() ) );

    emit infos(msg);
}

// file handling --------------------------------------------------------------------

void ImageView::load(const QString & file)
{
    m_image.load(file);
    setImage(m_image);
}

QImage& ImageView::getImage()
{
    return m_image;
}

// ImageOverview --------------------------------------------------------------------

ImageNavigator::ImageNavigator(QWidget* parent, Qt::WindowFlags flags) : QWidget(parent, flags),
    m_worldMatrix(NULL),
    m_imgMatrix(NULL),
    m_backgroundColor( QColor(0, 0, 0, 150) )
{
}

void ImageNavigator::paintEvent(QPaintEvent * /*event */)
{
    if (m_img.isNull() || !m_imgMatrix || !m_worldMatrix)
        return;

    QPainter painter(this);

    QRectF imgRect = QRectF( QPoint(), m_img.size() );
    QRectF overviewRect = QRectF( QPoint(), QSize(geometry().width() - 1, geometry().height() - 1) );         // get the overview rect

    QTransform overviewImgMatrix = getScaledImageMatrix();          // matrix that always resizes the image to the current viewport
    QRectF overviewImgRect = overviewImgMatrix.mapRect(imgRect);
    overviewImgRect.setTop(overviewImgRect.top() + 1);
    overviewImgRect.setLeft(overviewImgRect.left() + 1);
    overviewImgRect.setWidth(overviewImgRect.width() - 1);            // we have a border... correct that...
    overviewImgRect.setHeight(overviewImgRect.height() - 1);

    // now render the current view
    QRectF viewRect = m_viewPortRect;
    viewRect = m_worldMatrix->inverted().mapRect(viewRect);
    viewRect = m_imgMatrix->inverted().mapRect(viewRect);
    viewRect = overviewImgMatrix.mapRect(viewRect);

    //draw the image's location
    painter.setBrush(m_backgroundColor);
    painter.setPen( QColor(200, 200, 200) );
    painter.drawRect(overviewRect);
    painter.setOpacity(0.8f);
    painter.drawImage( overviewImgRect, m_imgT, QRect( 0, 0, m_imgT.width(), m_imgT.height() ) );

    painter.setPen( QColor(100, 0, 0) );
    painter.setBrush( QColor(100, 0, 0, 50) );
    painter.drawRect(viewRect);
}

void ImageNavigator::mousePressEvent(QMouseEvent *event)
{
    m_enterPos = event->pos();
    m_lastPos = event->pos();
}

void ImageNavigator::mouseReleaseEvent(QMouseEvent *event)
{
    QPointF dxy = m_enterPos - QPointF( event->pos() );

    if (dxy.manhattanLength() < 4)
    {
        // move to the current position
        // now render the current view
        QRectF viewRect = m_viewPortRect;
        viewRect = m_worldMatrix->inverted().mapRect(viewRect);
        viewRect = m_imgMatrix->inverted().mapRect(viewRect);
        viewRect = getScaledImageMatrix().mapRect(viewRect);
        QPointF currentViewPoint = viewRect.center();

        float panningSpeed = -( m_worldMatrix->m11() / ( getScaledImageMatrix().m11() / m_imgMatrix->m11() ) );

        QPointF cPos = event->pos();
        QPointF dxy = (cPos - currentViewPoint) / m_worldMatrix->m11() * panningSpeed;
        emit moveViewSignal(dxy);
    }
}

void ImageNavigator::mouseMoveEvent(QMouseEvent *event)
{
    if (event->buttons() != Qt::LeftButton)
        return;

    float panningSpeed = -( m_worldMatrix->m11() / ( getScaledImageMatrix().m11() / m_imgMatrix->m11() ) );

    QPointF cPos = event->pos();
    QPointF dxy = (cPos - m_lastPos) / m_worldMatrix->m11() * panningSpeed;
    m_lastPos = cPos;
    emit moveViewSignal(dxy);
}

void ImageNavigator::resizeEvent(QResizeEvent* event)
{
    if ( event->size() == size() )
        return;

    resizeImg();

    QWidget::resizeEvent(event);
}

void ImageNavigator::resize(int w, int h)
{
    resize( QSize(w, h) );
}

void ImageNavigator::resize(const QSize& size)
{
    QWidget::resize(size);

    // update image
    resizeImg();
}

void ImageNavigator::resizeImg()
{
    if ( m_img.isNull() )
        return;

    QRectF imgRect = QRectF( QPoint(), m_img.size() );

    QTransform overviewImgMatrix = getScaledImageMatrix();          // matrix that always resizes the image to the current viewport
    QRectF overviewImgRect = overviewImgMatrix.mapRect(imgRect);
    overviewImgRect.setTop(overviewImgRect.top() + 1);
    overviewImgRect.setLeft(overviewImgRect.left() + 1);
    overviewImgRect.setWidth(overviewImgRect.width() - 1);            // we have a border... correct that...
    overviewImgRect.setHeight(overviewImgRect.height() - 1);

    // fast downscaling
    m_imgT = m_img.scaled(overviewImgRect.size().width(), overviewImgRect.size().height(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
}

QTransform ImageNavigator::getScaledImageMatrix()
{
    if ( m_img.isNull() )
        return QTransform();

    // the image resizes as we zoom
    QRectF imgRect = QRectF( QPoint(), m_img.size() );
    float ratioImg = imgRect.width() / imgRect.height();
    float ratioWin = (float)width() / (float)height();

    QTransform imgMatrix;
    float s;

    if (imgRect.width() == 0 || imgRect.height() == 0)
        s = 1.0f;
    else
        s = (ratioImg > ratioWin) ? (float)width() / imgRect.width() : (float)height() / imgRect.height();

    imgMatrix.scale(s, s);

    QRectF imgViewRect = imgMatrix.mapRect(imgRect);
    imgMatrix.translate( ( width() - imgViewRect.width() ) * 0.5f / s, ( height() - imgViewRect.height() ) * 0.5f / s );

    return imgMatrix;
}
