
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QTimer>

#include "Scheduler.h"
#include "AppSettings.h"

#include "LogHandler.h"

const int minInterval = 30; // 30 sec

Scheduler::Scheduler(QObject *parent)
    : QObject(parent),
    m_changeAtStartup(false),
    m_quitAfterChange(false),
    m_intervalBetweenChangeMode(false),
    m_changeByInterval(true),
    m_changeInterval( QTime(0,30) ), // 30 min by default
    m_idle(true)
{
    loadSettings();

    m_eventTimer.setSingleShot(true);
    connect( &m_eventTimer, SIGNAL( timeout() ), this, SLOT( newSchedule() ) );
    m_quitTimer.setSingleShot(true);
    m_quitTimer.setInterval(minInterval * 1000);
    connect( &m_quitTimer, SIGNAL( timeout() ), this, SIGNAL( quit() ) );
}

Scheduler::~Scheduler()
{
    //saveSettings();
}

void Scheduler::start()
{
    m_idle = false;
    checkInitState();
}

void Scheduler::checkInitState()
{
    if (m_changeAtStartup && !m_idle)
    {
        QTimer::singleShot( 1000, this, SLOT( newSchedule() ) );
        LogHandler::getInstance()->reportDebug("Change at Startup ...");

        if (m_quitAfterChange)
            m_quitTimer.start();
    }

    setupNextInterval();
}

void Scheduler::stopQuitAfterChange()
{
    if (m_quitAfterChange)
    {
        LogHandler::getInstance()->reportDebug("Auto Quit application is disabled for this session ...");
        m_quitTimer.stop();
    }
}

void Scheduler::setupNextInterval()
{
    if (!m_intervalBetweenChangeMode || m_idle)
        return;

    m_referenceTime = QDateTime::currentDateTime();

    if (m_changeByInterval) // stable interval between each event
    {
        int count = -m_changeInterval.secsTo( QTime(0,0,0) );
        if (count < minInterval)
        {
            LogHandler::getInstance()->reportDebug( QString("update interval is to low (%1) => force to %2 sec ...").arg(count).arg(minInterval) );
            count = minInterval;
        }
        m_eventTimer.start(count * 1000);
        LogHandler::getInstance()->reportDebug( QString("Start new timer for %1 sec ...").arg(count) );
    }
    else // variable interval (list of Time Of Day)
    {
        // search for next valid time
        QTime currentTime = QTime::currentTime();
        foreach (const QTime &time, m_listTimeOfDay)
        {
            if (time > currentTime)
            {
                int count = qMax(currentTime.secsTo(time),1);
                m_eventTimer.start(++count * 1000);
                LogHandler::getInstance()->reportDebug( QString("Start new timer for %1 sec ...").arg(count) );
                break;
            }
        }
    }
}

void Scheduler::newSchedule()
{
    if (m_idle)
        return;

    LogHandler::getInstance()->reportDebug( QString("New Schedule fired ...") );
    emit newEvent();
    setupNextInterval();
}

void Scheduler::restartScheduler()
{
    setupNextInterval();
}

void Scheduler::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("Scheduler");

    m_changeAtStartup = settings.value("changeAtStartup",false).toBool();
    m_quitAfterChange = settings.value("quitAfterChange",false).toBool();
    m_intervalBetweenChangeMode = settings.value("intervalBetweenChangeMode",false).toBool();

    m_changeByInterval = settings.value("changeByInterval",true).toBool();

    m_changeInterval = settings.value("changeInterval","").toTime();

    int size = settings.beginReadArray("listTimeOfDay");
    for (int i = 0; i < size; ++i)
    {
        settings.setArrayIndex(i);
        m_listTimeOfDay << settings.value("tod").toTime();
    }
    settings.endArray();

    settings.endGroup();
}

void Scheduler::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("Scheduler");

    settings.setValue("changeAtStartup", m_changeAtStartup);
    settings.setValue("quitAfterChange", m_quitAfterChange);
    settings.setValue("intervalBetweenChangeMode", m_intervalBetweenChangeMode);

    settings.setValue("changeByInterval", m_changeByInterval);

    settings.setValue("changeInterval", m_changeInterval);

    settings.beginWriteArray( "listTimeOfDay", m_listTimeOfDay.count() );
    for (int i = 0; i < m_listTimeOfDay.size(); ++i)
    {
        settings.setArrayIndex(i);
        settings.setValue( "tod", m_listTimeOfDay.at(i) );
    }
    settings.endArray();

    settings.endGroup();
}

void Scheduler::setChangeByInterval(bool val)
{
    m_changeByInterval = val;
    setupNextInterval();
}

void Scheduler::setChangeAtStartup(bool val)

{
    m_changeAtStartup = val;
}

void Scheduler::setQuitAfterChange(bool val)
{
    m_quitAfterChange = val;
}

void Scheduler::setIntervalBetweenChangeMode(bool val)
{
    m_intervalBetweenChangeMode = val;
    setupNextInterval();
}

void Scheduler::setInterval(const QTime &time)
{
    m_changeInterval = time;
    setupNextInterval();
}

bool Scheduler::addTOD(const QTime &tod)
{
    if (m_listTimeOfDay.indexOf(tod) != -1)
        return false;

    m_listTimeOfDay << tod;
    qSort(m_listTimeOfDay);
    setupNextInterval();
    saveSettings();
    return true;
}

bool Scheduler::deleteTOD(const QTime &tod)
{
    bool ok = m_listTimeOfDay.removeOne(tod);
    if (ok)
        setupNextInterval();
    saveSettings();
    return ok;
}

void Scheduler::setIdle(bool val)
{
    m_idle = val;
    if (!m_idle)
        setupNextInterval();
}
