
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include "ScreenItem.h"
#include <QtGui/QPainter>
#include <QtGui/QStyleOptionGraphicsItem>

const int decoMargin = 2; // default pen width + supplementary contour if selected (so 1 + 1)
const int penWidth = 1;
const int cornerSize = 2;
const int fontFactor = 15;

ScreenItem::ScreenItem ( const QSize &size, int num, QGraphicsItem * parent ) : QGraphicsItem(parent),
    m_realSize(size),
    m_size(size),
    m_screenNum(num),
    m_marginLeft(70),
    m_marginRight(70),
    m_marginTop(0),
    m_marginBottom(0),
    m_useMargins(true),
    m_drawText(true),
    m_opaque(true)
{
    setFlag(QGraphicsItem::ItemIsMovable, true);
    setFlag(QGraphicsItem::ItemIsSelectable, true);
    setFlag(QGraphicsItem::ItemIsFocusable, true);
}

void ScreenItem::setSize(const QSize &size)
{
    m_realSize = size;
    m_size = size;
}

void ScreenItem::setNumber(int num)
{
    m_screenNum = num;
}

QSize ScreenItem::getRealSizeWithMargins() const
{
    QSize size(m_realSize);
    if (m_useMargins)
    {
        size.rwidth() += (m_marginLeft + m_marginRight);
        size.rheight() += (m_marginTop + m_marginBottom);
    }
    return size;
}

QSize ScreenItem::getSizeWithMargins() const
{
    QSize size(m_size);
    if (m_useMargins)
    {
        size.rwidth() += (m_marginLeft + m_marginRight);
        size.rheight() += (m_marginTop + m_marginBottom);
    }
    return size;
}

QRectF ScreenItem::boundingRect() const
{
    QRectF res;
    if (m_useMargins)
        res = QRectF(
            -(m_size.width() + m_marginLeft + m_marginRight  ) / 2 - decoMargin,
            -(m_size.height() + m_marginTop + m_marginBottom) / 2 - decoMargin,
            m_size.width() + m_marginLeft + m_marginRight + 2 * decoMargin,
            m_size.height() + m_marginTop + m_marginBottom + 2 * decoMargin
            );
    else
        res = QRectF(
            -( m_size.width() ) / 2 - decoMargin,
            -( m_size.height() ) / 2 - decoMargin,
            m_size.width() + 2 * decoMargin,
            m_size.height() + 2 * decoMargin
            );

    return res;
}

QRectF ScreenItem::getPreviewSceneWithMarginsBoundingRect()
{
    QRectF sceneRect = sceneBoundingRect();
    QRectF res;
    if (m_useMargins)
        res = QRectF (
            (sceneRect.x() + (m_marginLeft + m_marginRight) / 2 + decoMargin) / 10.0f,
            (sceneRect.y() + (m_marginTop + m_marginBottom) / 2 + decoMargin) / 10.0f,
            m_size.width() / 10.0f,
            m_size.height() / 10.0f
            );
    else
        res = QRectF (
            (sceneRect.x() / 2 + decoMargin) / 10.0f,
            (sceneRect.y() / 2 + decoMargin) / 10.0f,
            m_size.width() / 10.0f,
            m_size.height() / 10.0f
            );
    return res;
}

QRectF ScreenItem::getPreviewSceneBoundingRect()
{
    QRectF sceneRect = sceneBoundingRect();
    return QRectF (
               (sceneRect.x() + decoMargin) / 10.0f,
               (sceneRect.y() + decoMargin) / 10.0f,
               m_size.width() / 10.0f,
               m_size.height() / 10.0f
               );
}

QRectF ScreenItem::getValidSceneBoundingRect()
{
    QRectF sceneRect = sceneBoundingRect();
    QRectF res;

    if (m_useMargins)
        res = QRectF (
            sceneRect.x() + (m_marginLeft + m_marginRight) / 2 + decoMargin,
            sceneRect.y() + (m_marginTop + m_marginBottom) / 2 + decoMargin,
            m_size.width(),
            m_size.height()
            );
    else
        res = QRectF (
            sceneRect.x() / 2 + decoMargin,
            sceneRect.y() / 2 + decoMargin,
            m_size.width(),
            m_size.height()
            );
    return res;
}

void ScreenItem::setImage(const QImage &image)
{
    m_image = image;
    m_cropimage = m_image.copy( getPreviewSceneWithMarginsBoundingRect().toRect() );
    update();
}

void ScreenItem::updateImage()
{
    m_cropimage = m_image.copy( getPreviewSceneWithMarginsBoundingRect().toRect() );
    update();
}

void ScreenItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget * /* widget */)
{
    QFont font;
    font.setPixelSize(10 * fontFactor);
    painter->setFont(font);

    QPen pen(Qt::black);
    pen.setWidth(1);
    pen.setCosmetic(true);

    //draw out box
    pen.setBrush(Qt::black);
    painter->setPen(pen);

    QRectF itemRect = boundingRect().translated(decoMargin - penWidth,decoMargin - penWidth);

    painter->drawRoundedRect(itemRect,cornerSize,cornerSize);

    // draw borders
    if (m_useMargins)
        drawMargins(painter);

    // draw image
    drawDesktop(painter);

    // draw screen number
    if (m_drawText)
        drawTextEffect( painter, QString::number(m_screenNum),itemRect.translated(9 * fontFactor,9 * fontFactor), Qt::AlignLeft, QPoint(fontFactor, fontFactor) );

    // display resolution
    if (m_drawText)
    {
        font.setPixelSize(5 * fontFactor);
        painter->setFont(font);
        QString res = QString("%1x%2").arg( m_realSize.width() ).arg( m_realSize.height() );
        drawTextEffect( painter, res,itemRect.translated(9 * fontFactor,itemRect.height() - 20 * fontFactor), Qt::AlignLeft, QPoint(fontFactor / 2, fontFactor / 2) );
    }

    // draw highlight if needed
    if (option->state & QStyle::State_Selected)
    {
        pen.setBrush(Qt::red);
        painter->setPen(pen);
        painter->drawRoundedRect(itemRect, cornerSize, cornerSize);
    }
}

void ScreenItem::drawTextEffect(QPainter *painter, const QString &text, const QRectF & rect, int alignment, const QPoint &offset)
{
    // Draw shadow.
    painter->setPen( QPen(Qt::gray) );
    painter->drawText(rect.translated(offset), alignment, text);

    // Draw text.
    painter->setPen( QPen(Qt::black) );
    painter->drawText(rect, alignment, text);
}

void ScreenItem::drawDesktop(QPainter *painter)
{
    if ( m_image.isNull() )
    {
        painter->fillRect (
            -m_size.width() / 2,
            -m_size.height() / 2,
            m_size.width(),
            m_size.height(),
            Qt::white );
    }
    else
    {
        QRectF target(
            -m_size.width() / 2,
            -m_size.height() / 2,
            m_size.width(),
            m_size.height()
            );
        if (m_opaque)
            painter->drawImage( target,m_cropimage,m_cropimage.rect() );

        // add a reflextion effect just for fun !!
        QImage reflection(":/syncwall/reflection.png");
        painter->drawImage( target, reflection, reflection.rect() );
    }
}

void ScreenItem::drawMargins(QPainter *painter)
{
    int x0 = -(m_size.width() + m_marginLeft + m_marginRight) / 2;
    int y0 = -(m_size.height() + m_marginTop + m_marginBottom) / 2;

    if (m_marginTop)
    {
        painter->fillRect (
            x0,
            y0,
            m_size.width() + m_marginLeft + m_marginRight,
            m_marginTop,
            Qt::black );
    }

    if (m_marginBottom)
    {
        painter->fillRect (
            x0,
            y0 + m_marginTop + m_size.height(),
            m_size.width() + m_marginLeft + m_marginRight,
            m_marginBottom,
            Qt::black );
    }

    if (m_marginRight)
    {
        painter->fillRect (
            x0 + m_marginLeft + m_size.width(),
            y0,
            m_marginRight,
            m_size.height() + m_marginTop + m_marginBottom,
            Qt::black );
    }

    if (m_marginLeft)
    {
        painter->fillRect (
            x0,
            y0,
            m_marginLeft,
            m_size.height() + m_marginTop + m_marginBottom,
            Qt::black );
    }
}

void ScreenItem::setMargings(int top, int bottom, int left, int right)
{
    prepareGeometryChange();
    m_marginTop = top;
    m_marginBottom = bottom;
    m_marginLeft = left;
    m_marginRight = right;
    update();
}

void ScreenItem::setMargingTop(int val)
{
    prepareGeometryChange();
    m_marginTop = val;
    update();
}

void ScreenItem::setMargingBottom(int val)
{
    prepareGeometryChange();
    m_marginBottom = val;
    update();
}

void ScreenItem::setMargingLeft(int val)
{
    prepareGeometryChange();
    m_marginLeft = val;
    update();
}

void ScreenItem::setMargingRight(int val)
{
    prepareGeometryChange();
    m_marginRight = val;
    update();
}

void ScreenItem::setUseMargins(bool val)
{
    m_useMargins = val;
    update();
}

void ScreenItem::setDrawText(bool val)
{
    m_drawText = val;
    update();
}

void ScreenItem::setOpaque(bool val)
{
    m_opaque = val;
    update();
}
