
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include "SyncWall.h"

#include <QtCore/QTranslator>

#include <QtGui/QDesktopWidget>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <QtGui/QDesktopServices>

#include <QtSql/QSqlDatabase>

#include <algorithm>
#include <vector>

#include "LogHandler.h"

#include "AppSettings.h"
#include "WallpaperProvider.h"

#include "MiscFunctions.h"
#include "DesktopScene.h"
#include "ImageViewer.h"
#include "ListImagesModel.h"
#include "TreeImagesView.h"
#include "IconViewDelegate.h"
#include "NetworkManager.h"

#include "ThreadPool.h"
#include "StarDelegate.h"

#include <stdio.h>

#ifdef Q_WS_WIN // for wallpaper set on windows platform

#include <windows.h>
#include <shlobj.h>

#endif

#ifndef Q_WS_PM // OS/2
#include <QxtGlobalShortcut>
#endif

#include "ui_SyncWall.h"

const int defaultBlinkInterval = 1000; // 1 sec
const int defaultBlinkingTime = 5000; // 5 sec
const int saveSettingsInterval = 10 * 60 * 1000; // 10 min

SyncWall::SyncWall(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags),
    ui(new Ui::SyncWallClass),
    m_useGlobalHotKey(false),
    m_globalHotkeyNext("Ctrl+F11"),
    m_globalHotkeyPrev("Ctrl+F12"),
    m_globalHotkeyOpen("Ctrl+F10"),
    m_globalHotkeyOffset("Ctrl+F9"),
#ifndef Q_WS_PM // OS/2
    m_shortCutNext(NULL),
    m_shortCutPrev(NULL),
    m_shortCutOpen(NULL),
    m_shortCutOffset(NULL),
#endif
    m_firstCloseMessage(false),
    m_appName(PACKAGE_NAME),
    m_version(PACKAGE_VERSION),
    m_trayIcon(NULL),
    m_scheduler(NULL),
    m_logLevel(LogHandler::MSG_DEBUG),
    m_randomOrderChange(false),
    m_currentImageIndex(-1),
    m_blinking(true),
    m_startInTray(false),
    m_splashscreenAtStartup(true),
    m_splashscreenTransparentBackground(true),
    m_startWithSystem(false),
    m_listModel(NULL),
    m_resetConfig(false),
    m_currentListIndex(-1)
{
    ui->setupUi(this);

    // load the settings
    loadSettings();

    // create/connect specific class/ui->widget and setup various parameters
    updateUi();

    // update wallpaper file list
    refreshWallpaperProviders();
}

SyncWall::~SyncWall()
{
    saveSettings();
    ThreadPool::getInstance()->stop();
    qDeleteAll(m_currentProviders);
    delete ui;
}

void SyncWall::updateUi()
{
    //checkForSmallInterface();

    initStateDefs();

    // setup app name, release, ...
    updateApplicationIdentity();

    // setup About dialog with some informations
    updateAbout();

    // update language data
    updatePrefsLanguage();

    // create and connect the log handler
    connect( LogHandler::getInstance(),SIGNAL( newMessage(const QString &) ),this,SLOT( printToLog(const QString &) ) );
    LogHandler::getInstance()->setBufferization(false); // get all stored messages if exists

    // force always tab(0)
    ui->tabWidgetMain->setCurrentIndex(0);
    ui->tabWidgetPreferences->setCurrentIndex(0);

    createActionsSystemTray();
    createTrayIcon();

    // limit log display
    ui->textBrowserLog->document()->setMaximumBlockCount(1000);

    // init random generator to a "random" value !! :)
    qsrand( QTime::currentTime().msec() );

    // temp storage check
    QString tmpPath = MiscFunctions::getValidStorage();

    if ( !QDir(tmpPath).exists() )
    {
        if ( !QDir::home().mkpath(tmpPath) )
            QMessageBox::warning( this,m_appName,tr("Can't create storage location, check why ?! (%1)").arg(tmpPath) );
    }

#ifdef Q_WS_WIN

    SHELLFLAGSTATE shfs;

    SHGetSettings(&shfs,SSF_DESKTOPHTML);

    if (shfs.fDesktopHTML)
        QMessageBox::warning( this,m_appName,tr("Active Desktop must be disabled!!") );

//#else
    //ui->checkBoxStartWithSystem->hide(); // hide autostart checkbox only usable on Windows
#endif

    // connect network manager signals
    connect( ui->widgetNetworkParameters, SIGNAL( newState() ), this, SLOT( checkState() ) );
    connect( ui->widgetNetworkParameters, SIGNAL( newMessage(const QString & ) ), ui->widgetImageParameters, SLOT( decodeAndApplyMessage(const QString & ) ) );
    connect( ui->widgetNetworkParameters, SIGNAL( newMessage(const NetworkMessage & ) ), ui->widgetImageParameters, SLOT( decodeAndApplyMessage(const NetworkMessage & ) ) );
    connect( ui->widgetNetworkParameters, SIGNAL( synchronizeByNetwork(bool) ), ui->widgetScheduler, SLOT( setIdle(bool) ) );
    connect( this, SIGNAL( sigSaveSettings() ), ui->widgetNetworkParameters, SLOT( saveSettings() ) );

    connect( ui->widgetImageParameters, SIGNAL( requestNewImage() ), this, SLOT( changeWallpaper() ) );
    connect( ui->widgetImageParameters, SIGNAL( decodedImage(const QString & ) ), this, SLOT( searchForWallpaper(const QString & ) ) );
    connect( ui->widgetImageParameters, SIGNAL( newNetworkImage(const QString & ) ), this, SLOT( forceWallpaper(const QString & ) ) );
    connect( ui->widgetImageParameters, SIGNAL( parametersChange() ), this, SLOT( changeMonitorSupport() ) );
    connect( this, SIGNAL( sigSaveSettings() ), ui->widgetImageParameters, SLOT( saveSettings() ) );

    // connect scheduler signals
    connect( ui->widgetScheduler, SIGNAL( newState() ), this, SLOT( checkState() ) );
    connect( ui->widgetScheduler, SIGNAL( newEvent() ), this, SLOT( checkForBlinking() ) );
    connect( ui->widgetScheduler, SIGNAL( quit() ), this, SLOT( on_pushButtonQuit_pressed() ) );
    connect( this, SIGNAL( sigSaveSettings() ), ui->widgetScheduler, SLOT( saveSettings() ) );

    // starting network before scheduler
//    ui->widgetNetworkParameters->start();
    ui->widgetScheduler->start();

    // setup "blinking" timer
    m_blinkTimer.setInterval(defaultBlinkInterval);
    connect( &m_blinkTimer, SIGNAL( timeout() ), this, SLOT( blink() ) );

    // connect painter to system integration
    connect( ui->widgetImageParameters, SIGNAL( newImage(const QImage &, bool) ), ui->widgetSystem, SLOT( applyWallpaper(const QImage &, bool) ) );
    connect( ui->widgetImageParameters, SIGNAL( newImages(const QList<QImage> &, bool) ), ui->widgetSystem, SLOT( applyWallpaper(const QList<QImage> &, bool) ) );

    // connect painter to network
    connect( ui->widgetImageParameters, SIGNAL( newConfiguration(const QString &) ), ui->widgetNetworkParameters, SLOT( sendNewMessage(const QString &) ) );
    connect( ui->widgetImageParameters, SIGNAL( newConfiguration(const NetworkMessage &) ), ui->widgetNetworkParameters, SIGNAL( sigSendNewMessage(const NetworkMessage &) ) );

    connect( this, SIGNAL( sigSaveSettings() ), ui->widgetSystem, SLOT( saveSettings() ) );

    ui->graphicsViewDesktop->setScene( ui->widgetImageParameters->getScenePreview() );

    // create the exclusive button
    QButtonGroup *buttonGroup = new QButtonGroup(this);
    buttonGroup->setExclusive(true);

    //buttonGroup->addButton(ui->pushButtonViewList);
    buttonGroup->addButton(ui->pushButtonViewDetail);
    buttonGroup->addButton(ui->pushButtonViewThumbnail);

    m_listModel = new ListImagesModel(this);

    // when infos are retrieve from database => must recompute index list to take into account rating
    connect ( m_listModel,SIGNAL( updateFinished() ), this, SLOT( computeIndexList() ) );

    StarDelegate *sd = new StarDelegate(this);
    sd->setStarColumn(ListImagesModel::COLUMN_RATE);
    ui->treeViewFiles->setItemDelegate(sd); // add specific star delegate delegate to view
    ui->treeViewFiles->setModel(m_listModel);

    ui->listViewFiles->setItemDelegate( new IconViewDelegate(this) ); // add specific icon delegate to view
    ui->listViewFiles->setModel(m_listModel);

    ui->stackedWidgetFiles->setCurrentIndex(0); // switch to treeView

    connect( ui->treeViewFiles->selectionModel(),SIGNAL ( selectionChanged (const QItemSelection &,  const QItemSelection & ) ), this, SLOT( selectionChanged(const QItemSelection &,  const QItemSelection & ) ) );
    connect( ui->listViewFiles->selectionModel(),SIGNAL ( selectionChanged (const QItemSelection &,  const QItemSelection & ) ), this, SLOT( selectionChanged(const QItemSelection &,  const QItemSelection & ) ) );

    // global hotkey not available for OS/2
#ifdef Q_WS_PM // OS/2
    ui->groupBoxHotKey->hide();
#endif

    // create a timer for automatically save settings
    QTimer * timer = new QTimer(this);
    connect( timer, SIGNAL( timeout() ), this, SLOT( saveSettings() ) );
    timer->start(saveSettingsInterval);

    // disable user move button if needed
    checkIfUserCanMove();

    // install event filter to receive keypress from view
    ui->listViewFiles->installEventFilter(this);
    ui->treeViewFiles->installEventFilter(this);

    // simple check of sql driver installed
    QStringList listDrivers( QSqlDatabase::drivers() );
    LogHandler::getInstance()->reportDebug( QString("Syncwall: available Sql drivers: %1").arg( listDrivers.join(", ") ) );
    if ( !listDrivers.contains("QSQLITE") && !listDrivers.contains("QSQLITE3") )
        QMessageBox::warning( this,m_appName,tr("The application can't find a valid SQLITE driver, its performance and functionality will be degraded") );
}

// on  laptop => we have to reduce the icon decoration size
void SyncWall::checkForSmallInterface()
{
    /*
       QRect screen = QApplication::desktop()->screenGeometry();
       qDebug() << screen;
       qDebug() << size();
       QSize size = minimumSizeHint () ;
       if (size.height() > screen.height()/2.0f)
       {
       // resize decoration icon
       QSize smallSize = ui->labelDecoration->pixmap()->size();
       smallSize /= 2;
       ui->labelDecoration->setPixmap(ui->labelDecoration->pixmap()->scaled(smallSize));
       ui->labelDecoration_2->setPixmap(ui->labelDecoration_2->pixmap()->scaled(smallSize));
       ui->labelDecoration_3->setPixmap(ui->labelDecoration_3->pixmap()->scaled(smallSize));
       ui->widgetNetworkParameters->labelDecoration->setPixmap(ui->widgetNetworkParameters->labelDecoration->pixmap()->scaled(smallSize));
       ui->widgetImageParameters->labelDecoration->setPixmap(ui->widgetImageParameters->labelDecoration->pixmap()->scaled(smallSize));
       ui->widgetScheduler->labelDecoration->setPixmap(ui->widgetScheduler->labelDecoration->pixmap()->scaled(smallSize));
       }
     */
}

void SyncWall::setupGlobalKeys()
{
#ifndef Q_WS_PM // OS/2
    // Next
    m_shortCutNext = new QxtGlobalShortcut(this);
    connect( m_shortCutNext, SIGNAL( activated() ), this, SLOT( on_actionNextWallpaper_activated() ) );
    QKeySequence keyNext(m_globalHotkeyNext);
    m_shortCutNext->setShortcut(keyNext);
    ui->widgetKeySequenceNext->setKeySequence(keyNext);
    ui->widgetKeySequenceNext->setClearButtonIcon( QIcon( (":/syncwall/delete.png") ) );

    // Prev
    m_shortCutPrev = new QxtGlobalShortcut(this);
    connect( m_shortCutPrev, SIGNAL( activated() ), this, SLOT( on_actionPreviousWallpaper_activated() ) );
    QKeySequence keyPrev(m_globalHotkeyPrev);
    m_shortCutPrev->setShortcut(keyPrev);
    ui->widgetKeySequencePrev->setKeySequence(keyPrev);
    ui->widgetKeySequencePrev->setClearButtonIcon( QIcon( (":/syncwall/delete.png") ) );

    // Open
    m_shortCutOpen = new QxtGlobalShortcut(this);
    connect( m_shortCutOpen, SIGNAL( activated() ), this, SLOT( on_actionShowUI_triggered() ) );
    QKeySequence keyOpen(m_globalHotkeyOpen);
    m_shortCutOpen->setShortcut(keyOpen);
    ui->widgetKeySequenceOpen->setKeySequence(keyOpen);
    ui->widgetKeySequenceOpen->setClearButtonIcon( QIcon( (":/syncwall/delete.png") ) );

    // Offset
    m_shortCutOffset = new QxtGlobalShortcut(this);
    connect( m_shortCutOffset, SIGNAL( activated() ), this, SLOT( on_pushButtonUserMove_pressed() ) );
    QKeySequence keyOffset(m_globalHotkeyOffset);
    m_shortCutOffset->setShortcut(keyOffset);
    ui->widgetKeySequenceOffset->setKeySequence(keyOffset);
    ui->widgetKeySequenceOffset->setClearButtonIcon( QIcon( (":/syncwall/delete.png") ) );
#endif
}

void SyncWall::updatePrefsLanguage()
{
    // set detected language
    QString detectedLanguage = QLocale::system().name().left(2);
    ui->labelDetectedLanguage->setText( tr("detected language: %1").arg(detectedLanguage) );

    // fill ui->comboBox
    const QMap<QString, QString> &mapLang = MiscFunctions::getAvailableLanguages();
    QStringList names = mapLang.keys();
    ui->comboBoxLanguage->addItem(tr("Automatic detection"), "auto");
    if (m_currentLanguage == "auto")
        ui->comboBoxLanguage->setCurrentIndex(ui->comboBoxLanguage->count() - 1);
    ui->comboBoxLanguage->addItem(tr("Default (no use of translation files)"),"default");
    if (m_currentLanguage == "default")
        ui->comboBoxLanguage->setCurrentIndex(ui->comboBoxLanguage->count() - 1);
    foreach (const QString &lang, names)
    {
        ui->comboBoxLanguage->addItem(lang,mapLang[lang]);
        if (mapLang[lang] == m_currentLanguage)
            ui->comboBoxLanguage->setCurrentIndex(ui->comboBoxLanguage->count() - 1);
    }
}

void SyncWall::on_comboBoxLanguage_activated(int index)
{
    QString lang = ui->comboBoxLanguage->itemData(index).toString();
    if (lang == m_currentLanguage)
        return;

    m_currentLanguage = lang;

    // check if recursive scan
    int ret = QMessageBox::question(this, m_appName,
                                    tr("The application need to restart in order to take into account new translation\n, Do you want to restart application now?"),
                                    QMessageBox::Yes
                                    | QMessageBox::No);

    if (ret == QMessageBox::Yes)
        restart();
}

void SyncWall::restart()
{
    QProcess::startDetached( qApp->arguments()[0], QStringList() << qApp->arguments().mid(1) );
    QApplication::exit(0);
}

void SyncWall::on_pushButtonQuit_pressed()
{
    QApplication::exit(0);
}

void SyncWall::on_pushButtonHelp_pressed()
{
    QDesktopServices::openUrl( QUrl( QString(PACKAGE_ONLINE_HELP_URL) ) );
}

void SyncWall::on_pushButtonClearLog_pressed()
{
    ui->textBrowserLog->clear();
}

void SyncWall::on_pushButtonSaveLog_pressed()
{
    QString logFile = QFileDialog::getSaveFileName(
        this,
        tr("Log file"),
        m_lastDirectory,
        "Log file (*.log)");

    if ( logFile.isEmpty() )
        return;

    QFile file(logFile);
    if ( !file.open(QIODevice::WriteOnly | QIODevice::Text) )
    {
        LogHandler::getInstance()->reportError( tr("Can't save log file !!: %1").arg(logFile) );
    }

    QTextStream out(&file);
    out << ui->textBrowserLog->document()->toPlainText();
    file.close();
}

void SyncWall::on_pushButtonAddDirectory_pressed()
{
    QString dir = QFileDialog::getExistingDirectory(this, tr("Add Wallpaper Directory"),
                                                    m_lastDirectory,
                                                    QFileDialog::ShowDirsOnly
                                                    | QFileDialog::DontResolveSymlinks);

    if ( dir.isEmpty() )
        return;

    // check if recursive scan
    int rep = QMessageBox::question(this, m_appName,
                                    tr("Add all found directories recursively?"),
                                    QMessageBox::Yes
                                    | QMessageBox::No);

    bool recursiveSearch = (rep == QMessageBox::Yes);

    // check if watch at
    rep = QMessageBox::question(this, m_appName,
                                tr("Watch for modifications in these directories at application startup?"),
                                QMessageBox::Yes
                                | QMessageBox::No);

    bool watchForModification = (rep == QMessageBox::Yes);

    addDirToProviders(dir,recursiveSearch,watchForModification);
}

void SyncWall::addDirToProviders(const QString &dir, bool recursiveSearch, bool watchForModification)
{
    if ( dir.isEmpty() )
        return;

    m_lastDirectory = dir; // save current directory

    QStringList listDir;

    if (recursiveSearch)
        listDir = getSubDirectoryList( QDir(dir,"",QDir::Name,QDir::Dirs) );

    listDir << dir;

    int typeProvider = WallpaperProvider::MODE_FILELIST;
    if (watchForModification)
        typeProvider = WallpaperProvider::MODE_DIRECTORY;

    foreach (const QString &folder, listDir)
    {
        LogHandler::getInstance()->reportDebug( QString("add %1 folder").arg(folder) );
        WallpaperProvider *wp = new WallpaperProvider(folder,typeProvider);
        m_currentProviders << wp;
        updateDisplayedFileList(wp); // update UI
    }
}

void SyncWall::on_pushButtonAddFiles_pressed()
{
    QString formats = MiscFunctions::getAvailablesImageFormats();
    QStringList list = QFileDialog::getOpenFileNames(
        this,
        tr("Select one or more wallpapers"),
        m_lastDirectory,
        tr("Images (%1)").arg(formats) );

    if ( !list.isEmpty() )
    {
        m_lastDirectory = QFileInfo(list[0]).absolutePath(); // save current directory
        WallpaperProvider *wp = new WallpaperProvider(list);
        m_currentProviders << wp;
        updateDisplayedFileList(wp); // update UI
    }
}

void SyncWall::on_pushButtonRefresh_pressed()
{
    refreshWallpaperProviders(true);
}

//--------------------------------------------------------------------------------------
// LOG Functions
//--------------------------------------------------------------------------------------

void SyncWall::printToLog(const QString & mess)
{
    ui->textBrowserLog->append(mess);
}

//--------------------------------------------------------------------------------------

void SyncWall::on_pushButtonDeleteImage_pressed()
{
    on_actionDeleteImage_triggered();
}

//-----------------------------------------------------------------------
// tray icon functions
//-----------------------------------------------------------------------

void SyncWall::closeEvent(QCloseEvent *event)
{
    if ( m_trayIcon && m_trayIcon->isVisible() )
    {
        if (!m_firstCloseMessage)
        {
            QMessageBox::information( this, tr("Systray"),
                                      tr("The program will keep running in the "
                                         "system tray. To terminate the program, "
                                         "choose <b>Quit</b> in the context menu "
                                         "of the system tray entry.") );
            m_firstCloseMessage = true;
        }

        hide();
        event->ignore();
    }
}

void SyncWall::on_actionQuit_activated()
{
    on_pushButtonQuit_pressed();
}

void SyncWall::on_actionNextWallpaper_activated()
{
    changeWallpaper( getNextImageIndex(),true );
}

void SyncWall::on_actionPreviousWallpaper_activated()
{
    changeWallpaper( getPrevImageIndex(),true  );
}

void SyncWall::createActionsSystemTray()
{
    // nothing to do yet
}

void SyncWall::on_actionShowUI_triggered()
{
    show();
    activateWindow();

    // just in case
    ui->widgetScheduler->stopQuitAfterChange();
}

void SyncWall::createTrayIcon()
{
    // tray menu
    m_trayIconMenu = new QMenu(this);

    m_trayIconMenu->addAction(ui->actionShowUI);
    m_trayIconMenu->addAction(ui->actionNextWallpaper);
    m_trayIconMenu->addAction(ui->actionPreviousWallpaper);
    m_trayIconMenu->addAction(ui->actionUserOffset);
    m_trayIconMenu->insertSeparator(ui->actionQuit);
    m_trayIconMenu->addAction(ui->actionQuit);

    m_trayIconMenu->setTitle(m_appName);
    m_trayIconMenu->setIcon( QIcon(":/syncwall/logo_big.png") );

    m_trayIcon = new QSystemTrayIcon(this);
    m_trayIcon->setContextMenu(m_trayIconMenu);

    connect( m_trayIcon, SIGNAL( activated(QSystemTrayIcon::ActivationReason) ),
             this, SLOT( iconActivated(QSystemTrayIcon::ActivationReason) ) );

    m_trayIcon->setIcon( QIcon(":/syncwall/logo_big.png") );

    m_trayIcon->show();
}

//-----------------------------------------------------------------------
// Save/Restore parameters functions
//-----------------------------------------------------------------------

void SyncWall::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("MainWindow");

    settings.setValue( "MainWindowState",saveState(0) );

    settings.setValue( "size", size() );
    settings.setValue( "pos", pos() );

    settings.setValue( "fullScreen", isFullScreen() );

    // save the screen number for the splashsreeen ...
    if ( QApplication::desktop()->isVirtualDesktop() )
        settings.setValue( "screenNumber", QApplication::desktop()->screenNumber( pos() ) );
    else
        settings.setValue( "screenNumber", QApplication::desktop()->screenNumber(this) );

    settings.endGroup();

    settings.beginGroup("Application");

    settings.setValue("useGlobalHotKey", m_useGlobalHotKey);
    settings.setValue("globalHotkeyNext", m_globalHotkeyNext);
    settings.setValue("globalHotkeyPrev", m_globalHotkeyPrev);
    settings.setValue("globalHotkeyOpen", m_globalHotkeyOpen);
    settings.setValue("globalHotkeyOffset", m_globalHotkeyOffset);

    settings.setValue("randomOrderChange", m_randomOrderChange);

    settings.setValue("currentLanguage", m_currentLanguage);

    settings.setValue("resetConfig", m_resetConfig);

    // providers
    settings.beginWriteArray( "providers", m_currentProviders.count() );

    for (int i = 0; i < m_currentProviders.size(); ++i)
    {
        settings.setArrayIndex(i);
        settings.setValue( "mode", m_currentProviders.at(i)->getMode() );
        settings.setValue( "inputs", m_currentProviders.at(i)->getInputList() );
    }

    settings.endArray();

    settings.setValue("lastDirectory", m_lastDirectory);
    settings.setValue("lastFile", m_lastFile);
    settings.setValue("startInTray", m_startInTray);
    settings.setValue("splashscreenAtStartup", m_splashscreenAtStartup);
    settings.setValue("splashscreenTransparentBackground", m_splashscreenTransparentBackground);
    settings.setValue("startWithSystem", m_startWithSystem);

    settings.endGroup();

    // save child class settings
    emit sigSaveSettings();
}

void SyncWall::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("MainWindow");

    restoreState(settings.value("MainWindowState").toByteArray(), 0);

    resize( settings.value( "size", QSize(400, 400) ).toSize() );
    move( settings.value( "pos", QPoint(200, 200) ).toPoint() );

    // check if restore position isn't outside the current screen definitions (additional screen has been unplugged)
    if (QApplication::desktop()->screenNumber( pos() ) < 0)
        move( QPoint(200, 200) );

    bool fullScreen = settings.value("fullScreen",false).toBool();

    if (fullScreen)
        showFullScreen ();

    settings.endGroup();

    settings.beginGroup("Application");

    m_useGlobalHotKey = settings.value("useGlobalHotKey",false).toBool();

    m_globalHotkeyNext = settings.value("globalHotkeyNext","Ctrl+F11").toString();
    m_globalHotkeyPrev = settings.value("globalHotkeyPrev","Ctrl+F12").toString();
    m_globalHotkeyOpen = settings.value("globalHotkeyOpen","Ctrl+F10").toString();
    m_globalHotkeyOffset = settings.value("globalHotkeyOffset","Ctrl+F9").toString();

    // setup global key menu
    setupGlobalKeys();
    ui->groupBoxHotKey->setChecked(m_useGlobalHotKey);
    on_groupBoxHotKey_clicked(m_useGlobalHotKey);

    m_randomOrderChange = settings.value("randomOrderChange",false).toBool();
    ui->checkBoxRandomOrder->setChecked(m_randomOrderChange);

    m_resetConfig = settings.value("resetConfig", false).toBool();

    m_currentLanguage = settings.value("currentLanguage","auto").toString();

    if ( m_currentLanguage.isEmpty() )
        m_currentLanguage = QLocale::system().name().left(2);

    // providers
    int size = settings.beginReadArray("providers");

    for (int i = 0; i < size; ++i)
    {
        settings.setArrayIndex(i);
        int mode = settings.value("mode").toInt();

        if (mode == WallpaperProvider::MODE_DIRECTORY)
            m_currentProviders << new WallpaperProvider(settings.value("inputs").toStringList()[0]);
        else
            m_currentProviders << new WallpaperProvider( settings.value("inputs").toStringList() );
    }

    settings.endArray();

    m_lastDirectory = settings.value( "lastDirectory",QDesktopServices::storageLocation(QDesktopServices::HomeLocation) ).toString();
    m_lastFile = settings.value( "lastFile",QDesktopServices::storageLocation(QDesktopServices::HomeLocation) ).toString();

    m_startInTray = settings.value("startInTray",false).toBool();
    ui->checkBoxStartInTray->setChecked(m_startInTray);

    // start hidden => no message
    if (m_startInTray)
        m_firstCloseMessage = true;

    m_splashscreenAtStartup = settings.value("splashscreenAtStartup",true).toBool();
    ui->checkBoxSplashscreenAtStartup->setChecked(m_splashscreenAtStartup);

    m_splashscreenTransparentBackground = settings.value("splashscreenTransparentBackground",true).toBool();
    ui->checkBoxSplashscreenTransparentBackground->setChecked(m_splashscreenTransparentBackground);

    m_startWithSystem = settings.value("startWithSystem",false).toBool();
    ui->checkBoxStartWithSystem->setChecked(m_startWithSystem);

    settings.endGroup();
}

// preferences
void SyncWall::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason)
    {
        case QSystemTrayIcon::Trigger:
            break;

        case QSystemTrayIcon::DoubleClick:
            showNormal ();
            QWidget::setVisible(true);
            activateWindow();
            break;

        case QSystemTrayIcon::MiddleClick:

            //showMessage();
            break;

        default:
            ;
    }

    // activate action only if needed !!
    ui->actionUserOffset->setEnabled( ui->widgetImageParameters->isUserMoveAllowed() );
}

void SyncWall::refreshWallpaperProviders(bool force)
{
    //m_listModel->clear(); // clear ui->widget
    //m_listFiles.clear(); // clear global wallpaper list

    foreach (WallpaperProvider * wp, m_currentProviders)
    {
        if (force)
            wp->refreshWallpaperList();

        updateDisplayedFileList(wp);
    }

    // update the fixed list of index
    computeIndexList();
}

void SyncWall::updateDisplayedFileList(WallpaperProvider *wp)
{
    if (!wp)
        return;

    m_listModel->addImages( wp->getWallpaperList() ); // update global wallpaper list

    // update the "stats"
    ui->labelStats->setText( tr("%1 images in list").arg( m_listModel->rowCount() ) );
}

QStringList SyncWall::getSubDirectoryList(const QDir & dir)
{
    QStringList res;
    QFileInfoList qsl = dir.entryInfoList();
    foreach (const QFileInfo &entry, qsl)
    {
        if ( entry.isDir() )
        {
            QString dirname = entry.fileName();

            if ( (dirname==".") || (dirname == "..") )
                continue;

            res << entry.absoluteFilePath();
            res.append( getSubDirectoryList( QDir( entry.absoluteFilePath(),"",QDir::Name,QDir::Dirs) ) );
        }
    }
    return res;
}

void SyncWall::on_pushButtonSetWallpaper_pressed()
{
    on_actionSetWallpaper_triggered();
}

void SyncWall::checkForBlinking()
{
    if (!m_blinking) // direct wallpaper change
    {
        changeWallpaper();
    }
    else
    {
        m_blinkTimer.start();
        QTimer::singleShot( defaultBlinkingTime, this, SLOT( changeWallpaper() ) );
    }
}

void SyncWall::changeWallpaper(int index, bool manual)
{
    m_blinkTimer.stop();
    checkState(); // force correct icon

    if (m_listModel->rowCount() == 0)
        return;

    if (index < 0) // mode auto compute next wallpaper
        index = getNextImageIndex();

    m_currentImageIndex = index;

    // manual => reset the sheduler manually
    if (manual)
        ui->widgetScheduler->restartScheduler();

    // update user offset
    QPoint offset = m_listModel->getOffset(m_currentImageIndex);
    ui->widgetImageParameters->setUserOffset(offset);

    // send new wallpaper to connected client if  needed
    ui->widgetImageParameters->processImage(m_listModel->getFilename(m_currentImageIndex),manual);

    // change list style
    m_listModel->setLastAppliedRow(m_currentImageIndex);

    // update header
    updateApplicationIdentity();

    // track file in list view
    trackFileInView(m_currentImageIndex);
}

void SyncWall::trackFileInView(int ind)
{
    QModelIndex index = m_listModel->index(ind,0);

    // ensure line visible
    ui->treeViewFiles->scrollTo(index);
    ui->listViewFiles->scrollTo(index);

    // force selection to current index
    ui->treeViewFiles->selectionModel()->select(index,QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
    ui->listViewFiles->selectionModel()->select(index,QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
}

int SyncWall::getRandomIndex(int max)
{
    if (max > 1)
        return qrand() % (max);
    else
        return 0;
}

//! COmpute a list of random index
void SyncWall::computeIndexList()
{
    // bad location to do that !!!
    ui->treeViewFiles->resizeColumnToContents(0);

    m_listIndex.clear();
    m_currentListIndex = 0;

    if (!m_listModel)
        return;

    for (int i = 0; i < m_listModel->rowCount(); i++)
    {
        if (m_randomOrderChange)
        {
            // construct vector
            for (int index = 0; index < m_listModel->getRating(i); index++)
                m_listIndex.push_back(i);
        }
        else
        {
            m_listIndex.push_back(i);
        }
    }

    if (m_randomOrderChange)
        std::random_shuffle( m_listIndex.begin(), m_listIndex.end() );
}

int SyncWall::getNextImageIndex()
{
    if ( !m_listIndex.size() )
        return -1;

    m_currentListIndex++;
    m_currentListIndex = m_currentListIndex % m_listIndex.size();

    return m_listIndex[m_currentListIndex];
}

int SyncWall::getPrevImageIndex()
{
    if ( !m_listIndex.size() )
        return -1;

    m_currentListIndex--;
    if (m_currentListIndex < 0)
        m_currentListIndex = m_listIndex.size() - 1;

    return m_listIndex[m_currentListIndex];
}

void SyncWall::selectionChanged(const QItemSelection& selected, const QItemSelection& /*deselected*/) // update only the preview
{
    QModelIndexList indexes = selected.indexes();
    if ( indexes.isEmpty() )
        return;

    // take only first index for previewed file
    m_currPreviewImage = m_listModel->getFilename(indexes[0]);
    QPoint offset = m_listModel->getOffset(indexes[0]);

    ui->widgetImageParameters->setUserOffset(offset);

    // change list style
    m_listModel->setLastPreviewedRow( indexes[0].row() );
    ui->widgetImageParameters->previewImage(m_currPreviewImage);

    // force row selection if detail mode
    if (m_listModel->getDisplayMode() == ListImagesModel::MODE_DETAILS)
    {
        ui->treeViewFiles->selectionModel()->select(selected,QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
    }
}

void SyncWall::on_listViewFiles_doubleClicked(const QModelIndex & /*index*/)
{
    QModelIndexList selected = ui->listViewFiles->selectionModel()->selectedRows ();
    if ( !selected.isEmpty() )
    {
        // reset preview
        m_listModel->setLastPreviewedRow(-1);
        changeWallpaper(m_listModel->getIndex(selected[0]),true);
    }
}

void SyncWall::on_treeViewFiles_doubleClicked(const QModelIndex & /*index*/)
{
    // check if column != starrating column
    QModelIndexList selected = ui->treeViewFiles->selectionModel()->selectedIndexes ();
    if (!selected.isEmpty() && selected[0].column() != ListImagesModel::COLUMN_RATE)
    {
        changeWallpaper(m_listModel->getIndex(selected[0]),true);
    }
}

void SyncWall::on_pushButtonPreview_pressed()
{
    on_actionPreviewImage_triggered();
}

void SyncWall::on_actionPreviewImage_triggered()
{
    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedRows ();
    else
        selected = ui->listViewFiles->selectionModel()->selectedRows ();

    if ( !selected.isEmpty() )
    {
        ImageViewer *viewer = new ImageViewer( m_listModel->getFilename( selected.at(0) ) );
        viewer->show();
    }
}

void SyncWall::on_actionSetWallpaper_triggered()
{
    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedRows ();
    else
        selected = ui->listViewFiles->selectionModel()->selectedRows ();

    if ( !selected.isEmpty() )
    {
        m_currentImageIndex = m_listModel->getIndex(selected[0]);
        changeWallpaper(m_currentImageIndex,true);
    }
}

void SyncWall::on_actionDeleteImage_triggered()
{
    bool somethingToDo = false;

    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedRows ();
    else
        selected = ui->listViewFiles->selectionModel()->selectedRows ();

    foreach(const QModelIndex &index, selected)
    {
        QString file = m_listModel->getFilename(index);
        foreach (WallpaperProvider * wp, m_currentProviders)
        {
            wp->deleteInput(file);
        }
        somethingToDo = true;

        //m_listModel->removeRows( index.row(), 1) );
    }

    m_listModel->removeRows(selected);

    // reset the list
    if (somethingToDo)
    {
        // search for empty providers
        for (int i = m_currentProviders.size() - 1; i >= 0; i--)
            if ( m_currentProviders.at(i)->getWallpaperList().isEmpty() )
                delete m_currentProviders.takeAt(i);

        refreshWallpaperProviders();
    }
}

void SyncWall::on_checkBoxRandomOrder_stateChanged ( int state)
{
    m_randomOrderChange = state;

    // update the fixed list of index
    computeIndexList();
}

void SyncWall::updateApplicationIdentity()
{
    QString title = m_appName + " - " + m_version + " - " + m_listModel->getFilename(m_currentImageIndex);
    setWindowTitle(title);
}

void SyncWall::updateAbout()
{
    // setup credits ...
    updateCredits();

    // setup changelog ...
    updateChangelog();
    updateInfosImageFormats();
    updateInfosLanguages();
    ui->labelRelease->setText(m_appName + " " + m_version);
    ui->labelCheckRelease->setText( tr("<a href=\"%1\">Check for new release ...</a>").arg(PACKAGE_CHECK_RELEASE_URL) );

    ui->pushButtonPaypal->setBusinessId(PAYPAL_BUSINESS_ID);
    ui->pushButtonPaypal->setItemName( "Support TheHive" );
    ui->pushButtonPaypal->setItemId( "TheHive-DONATION" );
    ui->pushButtonPaypal->setCurrencyCode( "EUR" );
}

void SyncWall::updateInfosImageFormats()
{
    // update format list

    // clear the current list
    ui->listWidgetFormats->clear();

    QStringList formats = MiscFunctions::getAvailablesImageFormatsList();
    QMap<QString, QString> longFormats = MiscFunctions::getLongImageFormats();

    foreach (const QString &format, formats)
    {
        if ( longFormats.contains(format) )
            new QListWidgetItem(longFormats[format] + " (*." + format + ")", ui->listWidgetFormats);
        else
            new QListWidgetItem(format, ui->listWidgetFormats);
    }

    // Qt version
    ui->labelQtVersion->setText( tr("Qt version %1").arg(QT_VERSION_STR) );
}

void SyncWall::updateInfosLanguages()
{
// update translation list
    const QMap<QString, QString> &mapLang = MiscFunctions::getAvailableLanguages();
    QStringList names = mapLang.keys();
    ui->listWidgetTranslations->clear();

    QString currentLang = m_currentLanguage;

    if (currentLang.isEmpty() || currentLang == "auto")
        currentLang = QLocale::system().name().left(2);

    foreach (const QString &lang, names)
    {
        QString text;

        if (mapLang[lang] == currentLang)
            text = lang + " *";
        else
            text = lang;

        QListWidgetItem *item = new QListWidgetItem(text, ui->listWidgetTranslations);
        item->setData(Qt::UserRole,mapLang[lang]); // save the lang acronym
    }
}

void SyncWall::forceWallpaper(const QString &file)
{
    ui->widgetImageParameters->processImage( file, false, true );
}

void SyncWall::searchForWallpaper(const QString &file)
{
    LogHandler::getInstance()->reportInfo( QString("new file received: %1").arg(file) );

    for (int i = 0; i < m_listModel->rowCount(); i++)
        if ( m_listModel->getFilename(i).endsWith(file) )
        {
            m_currentImageIndex = i;
            ui->widgetImageParameters->processImage( m_listModel->getFilename(m_currentImageIndex) );
            return;
        }

    LogHandler::getInstance()->reportError( QString("Can't find local file as received: %1").arg(file) );
}

void SyncWall::blink()
{
    static int i = 0;

    if (i++ % 2)
        checkState();
    else
        m_trayIcon->setIcon( QIcon(":/syncwall/mode_idle.png") );
}

void SyncWall::changeTrayIcon()
{
    m_trayIcon->setIcon( QIcon(":/syncwall/mode_standard.png") );
    m_trayIcon->setIcon( QIcon(":/syncwall/mode_client_connected.png") );
    m_trayIcon->setIcon( QIcon(":/syncwall/mode_client_disconnected.png") );
    m_trayIcon->setIcon( QIcon(":/syncwall/mode_server.png") );
}

int SyncWall::getState()
{
    /*
       STATE_STANDARD = 0, // scheduling on, no connected client
       STATE_IDLE, // scheduling off, no connected client
       STATE_SERVER, // connected client
       STATE_CLIENT_CONNECTED, // connected to server
       STATE_CLIENT_DISCONNECTED, // can't connect to server
       STATE_CLIENT_INVALID // connected to server but invalid protocol (version ?)
     */
    int state = STATE_STANDARD;

    if (ui->widgetNetworkParameters->getNetworkMode() == NetworkManager::MODE_SERVER)
    {
        if (ui->widgetNetworkParameters->getClientNb() == 0) // no client
        {
            if ( ui->widgetScheduler->getIntervalBetweenChangeMode() ) // change by interval
                state = STATE_STANDARD;
            else
                state = STATE_IDLE;
        }
        else
            state = STATE_SERVER;
    }
    else if (ui->widgetNetworkParameters->getNetworkMode() == NetworkManager::MODE_CLIENT) // mode client
    {
        if (ui->widgetNetworkParameters->getState() != QAbstractSocket::UnconnectedState)
        {
            state = STATE_CLIENT_CONNECTED;

            // check  for invalid connection not yet defined !!
        }
        else
        {
            state = STATE_CLIENT_DISCONNECTED;
        }
    }
    return state;
}

void SyncWall::initStateDefs()
{
    m_mapToStateDesc[STATE_STANDARD] = tr("Standard state. Scheduling is activated");
    m_mapToStateDesc[STATE_IDLE] = tr("Idle state. No scheduling and no server connection");
    m_mapToStateDesc[STATE_SERVER] = tr("Server state. Al least a client is connected");
    m_mapToStateDesc[STATE_CLIENT_CONNECTED] = tr("Client state. Connected to server");
    m_mapToStateDesc[STATE_CLIENT_DISCONNECTED] = tr("Client state but disconnected from server");
    m_mapToStateDesc[STATE_CLIENT_INVALID] = tr("Client state but the protocol isn't correct. Upgrade client or server!");

    m_mapToStateIcon[STATE_STANDARD] = ":/syncwall/mode_standard.png";
    m_mapToStateIcon[STATE_IDLE] = ":/syncwall/mode_idle.png";
    m_mapToStateIcon[STATE_SERVER] = ":/syncwall/mode_server.png";
    m_mapToStateIcon[STATE_CLIENT_CONNECTED] = ":/syncwall/mode_client_connected.png";
    m_mapToStateIcon[STATE_CLIENT_DISCONNECTED] = ":/syncwall/mode_client_disconnected.png";
    m_mapToStateIcon[STATE_CLIENT_INVALID] = ":/syncwall/mode_client_invalid.png";
}

void SyncWall::checkState()
{
    int state = getState();

    // update UI labels
    ui->labelStateDesc->setText(m_mapToStateDesc[state]);
    ui->labelStateImg->setPixmap( QPixmap(m_mapToStateIcon[state]) );

    // update tray icon
    m_trayIcon->setIcon( QIcon(m_mapToStateIcon[state]) );
    setWindowIcon( QIcon(m_mapToStateIcon[state]) );
    m_trayIcon->setToolTip(m_mapToStateDesc[state]);

    // freeze interface if needed
    bool freeze = (state == STATE_CLIENT_CONNECTED ||
                   state == STATE_CLIENT_DISCONNECTED ||
                   state == STATE_CLIENT_INVALID);

    ui->listViewFiles->setEnabled(!freeze);
    ui->treeViewFiles->setEnabled(!freeze);
    ui->pushButtonAddDirectory->setEnabled(!freeze);
    ui->pushButtonAddFiles->setEnabled(!freeze);
    ui->pushButtonDeleteImage->setEnabled(!freeze);
    ui->pushButtonRefresh->setEnabled(!freeze);
    ui->pushButtonSetWallpaper->setEnabled(!freeze);
    ui->pushButtonPreview->setEnabled(!freeze);
    ui->pushButtonUserMove->setEnabled(!freeze);
    ui->checkBoxRandomOrder->setEnabled(!freeze);
    ui->pushButtonAddDirectory->setEnabled(!freeze);
    ui->pushButtonAddDirectory->setEnabled(!freeze);
    ui->pushButtonAddDirectory->setEnabled(!freeze);
    ui->pushButtonAddDirectory->setEnabled(!freeze);
}

void SyncWall::updateChangelog()
{
    fillTextedit("Changelog.txt", ui->textEditChangelog);
}

void SyncWall::updateCredits()
{
    fillTextedit("CREDITS.txt", ui->textEditCredits);
}

void SyncWall::fillTextedit(const QString &file, QTextEdit *text)
{
    text->viewport ()->setAutoFillBackground(false);
    QStringList paths;
    QString filePath;
    paths << QApplication::applicationDirPath () + "/" + file;
    paths << QApplication::applicationDirPath () + "/../" + file;
    paths << QApplication::applicationDirPath () + "/../../" + file;
    paths << QApplication::applicationDirPath () + "/../../../" + file;
    paths << QApplication::applicationDirPath () + "/../Resources/" + file; // MaxOSX
    paths << "/usr/share/" + QString("%1").arg(PACKAGE_NAME).toLower() + "/" + file;
    paths << "/usr/local/share/" + QString("%1").arg(PACKAGE_NAME).toLower() + "/" + file;

    foreach (const QString &path, paths)
    {
        if ( QFileInfo(path).exists() )
        {
            filePath = path;
            break;
        }
    }

    // perhaps, not found !!
    if ( QFileInfo(filePath).exists() )
    {
        QFile f( filePath );
        LogHandler::getInstance()->reportDebug( QString("%1 found").arg( filePath ) );
        if( f.open( QIODevice::ReadOnly | QIODevice::Text ) )
        {
            text->setText( f.readAll() );
            f.close();
            return;
        }
    }
    text->setText( tr("The %1 file can't be found, sorry ...").arg(file) );
}

void SyncWall::on_groupBoxHotKey_clicked(bool val)
{
#ifndef Q_WS_PM // OS/2
    m_useGlobalHotKey = val;
    m_shortCutNext->setEnabled(m_useGlobalHotKey);
    m_shortCutPrev->setEnabled(m_useGlobalHotKey);
    m_shortCutOpen->setEnabled(m_useGlobalHotKey);
    m_shortCutOffset->setEnabled(m_useGlobalHotKey);

    // Add shortcut to systray menu ui->action
    ui->actionNextWallpaper->setShortcuts( m_useGlobalHotKey ? QList<QKeySequence>() << m_shortCutNext->shortcut() : QList<QKeySequence>() );
    ui->actionPreviousWallpaper->setShortcuts( m_useGlobalHotKey ? QList<QKeySequence>() << m_shortCutPrev->shortcut() : QList<QKeySequence>() );
    ui->actionShowUI->setShortcuts( m_useGlobalHotKey ? QList<QKeySequence>() << m_shortCutOpen->shortcut() : QList<QKeySequence>() );
    ui->actionUserOffset->setShortcuts( m_useGlobalHotKey ? QList<QKeySequence>() << m_shortCutOffset->shortcut() : QList<QKeySequence>() );
#endif
}

void SyncWall::on_widgetKeySequenceNext_keySequenceChanged(const QKeySequence &seq)
{
#ifndef Q_WS_PM // OS/2
    m_shortCutNext->setShortcut(seq);
    m_globalHotkeyNext = seq.toString();
#endif
}

void SyncWall::on_widgetKeySequencePrev_keySequenceChanged(const QKeySequence &seq)
{
#ifndef Q_WS_PM // OS/2
    m_shortCutPrev->setShortcut(seq);
    m_globalHotkeyPrev = seq.toString();
#endif
}

void SyncWall::on_widgetKeySequenceOpen_keySequenceChanged(const QKeySequence &seq)
{
#ifndef Q_WS_PM // OS/2
    m_shortCutOpen->setShortcut(seq);
    m_globalHotkeyOpen = seq.toString();
#endif
}

void SyncWall::on_widgetKeySequenceOffset_keySequenceChanged(const QKeySequence &seq)
{
#ifndef Q_WS_PM // OS/2
    m_shortCutOffset->setShortcut(seq);
    m_globalHotkeyOffset = seq.toString();
#endif
}

void SyncWall::on_checkBoxStartInTray_stateChanged ( int state )
{
    m_startInTray = state;
}

void SyncWall::on_checkBoxSplashscreenAtStartup_stateChanged ( int state )
{
    m_splashscreenAtStartup = state;
}

void SyncWall::on_checkBoxSplashscreenTransparentBackground_stateChanged ( int state )
{
    m_splashscreenTransparentBackground = state;
}

void SyncWall::on_checkBoxStartWithSystem_stateChanged ( int state )
{
    m_startWithSystem = state;
    MiscFunctions::setRunOnStartup(m_startWithSystem);

/*
   #ifdef Q_WS_WIN
    QSettings regSettings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);

    if (m_startWithSystem)
        regSettings.setValue( PACKAGE_NAME,QDir::toNativeSeparators( QCoreApplication::applicationFilePath () ) );
    else
        regSettings.remove(PACKAGE_NAME);
   #endif
 */
}

void SyncWall::on_pushButtonViewDetail_pressed()
{
    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedIndexes ();
    else
        selected = ui->listViewFiles->selectionModel()->selectedIndexes ();

    m_listModel->setDisplayMode(ListImagesModel::MODE_DETAILS);
    ui->treeViewFiles->header()->setVisible(true);
    ui->treeViewFiles->resizeColumnToContents(0);

    ui->treeViewFiles->setFocus(Qt::MouseFocusReason);

    // restore selection
    if ( !selected.isEmpty() )
    {
        ui->treeViewFiles->selectionModel()->blockSignals(true);
        ui->treeViewFiles->selectionModel()->clearSelection();
        QSet<int> doneRow;
        foreach (const QModelIndex &index, selected)
        {
            if ( !doneRow.contains( index.row() ) )
            {
                ui->treeViewFiles->selectionModel()->select(m_listModel->index(index.row(),0),QItemSelectionModel::Select | QItemSelectionModel::Rows);
                doneRow << index.row();
            }
        }
        ui->treeViewFiles->selectionModel()->blockSignals(false);

        // scroll to first selection
        //ui->treeViewFiles->scrollTo(m_listModel->index(selected[0].row(),0));
    }

    ui->stackedWidgetFiles->setCurrentIndex(0); // switch to treeview
}

void SyncWall::on_pushButtonViewThumbnail_pressed()
{
    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedIndexes ();
    else
        selected = ui->listViewFiles->selectionModel()->selectedIndexes ();

    m_listModel->setDisplayMode(ListImagesModel::MODE_ICONS);

    ui->listViewFiles->setFocus(Qt::MouseFocusReason);

    // restore selection
    if ( !selected.isEmpty() )
    {
        ui->listViewFiles->selectionModel()->blockSignals(true);
        ui->listViewFiles->selectionModel()->clearSelection();
        QSet<int> doneRow;
        foreach (const QModelIndex &index, selected)
        {
            if ( !doneRow.contains( index.row() ) )
            {
                ui->listViewFiles->selectionModel()->select(m_listModel->index(index.row(),0),QItemSelectionModel::Select | QItemSelectionModel::Rows);
                doneRow << index.row();
            }
        }
        ui->listViewFiles->selectionModel()->blockSignals(false);

        // scroll to first selection
        //ui->listViewFiles->scrollTo(m_listModel->index(selected[0].row(),0));
    }

    ui->stackedWidgetFiles->setCurrentIndex(1); // switch to iconview
}

void SyncWall::dragEnterEvent(QDragEnterEvent *event)
{
    if ( event->mimeData()->hasUrls() )
        event->acceptProposedAction();
}

void SyncWall::dropEvent(QDropEvent *event)
{
    if ( event->mimeData()->hasUrls() )
    {
        QList<QUrl> urls = event->mimeData()->urls();
        QStringList files;
        QStringList dirs;
        for (int i = 0; i < urls.size(); ++i)
        {
            if ( QFileInfo( urls[i].toLocalFile() ).isDir() )
                dirs << urls[i].toLocalFile();
            else
                files << urls[i].toLocalFile();
        }

        // processing files in a single WallpaperProvider
        if ( !files.isEmpty() )
        {
            WallpaperProvider *wp = new WallpaperProvider(files);
            m_currentProviders << wp;
            updateDisplayedFileList(wp); // update UI
        }

        // processing directoriesfiles in a single WallpaperProvider
        if ( !dirs.isEmpty() )
        {
            // check if recursive scan
            int rep = QMessageBox::question(this, m_appName,
                                            tr("Add all found directories recursively?"),
                                            QMessageBox::Yes
                                            | QMessageBox::No);

            bool recursiveSearch = (rep == QMessageBox::Yes);

            // check if watch at
            rep = QMessageBox::question(this, m_appName,
                                        tr("Watch for modifications in these directories at application startup?"),
                                        QMessageBox::Yes
                                        | QMessageBox::No);

            bool watchForModification = (rep == QMessageBox::Yes);

            foreach(const QString &dir, dirs)
            addDirToProviders(dir,recursiveSearch,watchForModification);
        }
    }
}

void SyncWall::on_listViewFiles_customContextMenuRequested(const QPoint & pos)
{
    QModelIndex index = ui->listViewFiles->indexAt(pos);
    if ( index.isValid() )
    {
        QMenu menu(this);

        // if multi selection => only display the same value on each element
        QModelIndexList proxyList = ui->listViewFiles->selectionModel()->selectedRows ();

        ui->actionPreviewImage->setEnabled(false);
        ui->actionSetWallpaper->setEnabled(false);
        ui->actionDeleteImage->setEnabled(false);

        if (proxyList.size() > 0)
        {
            ui->actionPreviewImage->setEnabled(true);
            ui->actionSetWallpaper->setEnabled(true);
            ui->actionDeleteImage->setEnabled(true);
        }

        menu.addAction(ui->actionPreviewImage);
        menu.addAction(ui->actionSetWallpaper);
        menu.addAction(ui->actionDeleteImage);

        menu.exec( ui->listViewFiles->viewport()->mapToGlobal(pos) );
    }
}

void SyncWall::on_treeViewFiles_customContextMenuRequested(const QPoint & pos)
{
    QModelIndex index = ui->treeViewFiles->indexAt(pos);
    if ( index.isValid() )
    {
        QMenu menu(this);

        // if multi selection => only display the same value on each element
        QModelIndexList proxyList = ui->treeViewFiles->selectionModel()->selectedRows ();

        ui->actionPreviewImage->setEnabled(false);
        ui->actionSetWallpaper->setEnabled(false);
        ui->actionDeleteImage->setEnabled(false);

        if (proxyList.size() > 0)
        {
            ui->actionPreviewImage->setEnabled(true);
            ui->actionSetWallpaper->setEnabled(true);
            ui->actionDeleteImage->setEnabled(true);
        }

        menu.addAction(ui->actionPreviewImage);
        menu.addAction(ui->actionSetWallpaper);
        menu.addAction(ui->actionDeleteImage);

        menu.exec( ui->treeViewFiles->viewport()->mapToGlobal(pos) );
    }
}

void SyncWall::on_pushButtonResetPreferences_pressed()
{
    // check if recursive scan
    int ret = QMessageBox::question(this, m_appName,
                                    tr("Do you want to reset all the preferences to default value?\n If yes, the application will be relaunch"),
                                    QMessageBox::Yes
                                    | QMessageBox::No);

    if (ret == QMessageBox::Yes)
    {
        m_resetConfig = true;
        restart();
    }
}

void SyncWall::on_pushButtonResetCache_pressed()
{
    // check if recursive scan
    int ret = QMessageBox::question(this, m_appName,
                                    tr("Do you want to reset all cached data (thumbnails and ratings)?"),
                                    QMessageBox::Yes
                                    | QMessageBox::No);

    if (ret == QMessageBox::Yes)
        m_listModel->resetChache();
}

void SyncWall::wakeUp(const QString &mess)
{
    LogHandler::getInstance()->reportWarning(mess);
    show();
}

void SyncWall::enableBrowsing(bool val)
{
    ui->stackedWidgetFiles->setEnabled(val);
    ui->pushButtonAddDirectory->setEnabled(val);
    ui->pushButtonAddFiles->setEnabled(val);
    ui->pushButtonDeleteImage->setEnabled(val);
    ui->pushButtonRefresh->setEnabled(val);
    ui->pushButtonSetWallpaper->setEnabled(val);
    ui->pushButtonPreview->setEnabled(val);
}

void SyncWall::on_pushButtonUserMove_pressed()
{
    show(); // show the ui
    ui->pushButtonUserMove->setEnabled(false);
    ui->pushButtonUserValid->setEnabled(true);
    ui->pushButtonUserCancel->setEnabled(true);

    ui->widgetImageParameters->getScenePreview()->setSceneState(DesktopScene::STATE_MOVING_BACKGROUND);
    ui->graphicsViewDesktop->zoomToFit();
    enableBrowsing(false);
}

void SyncWall::on_pushButtonUserValid_pressed()
{
    ui->pushButtonUserMove->setEnabled(true);
    ui->pushButtonUserValid->setEnabled(false);
    ui->pushButtonUserCancel->setEnabled(false);

    QPointF offset = ui->widgetImageParameters->validCurrentUserPositioning();

    QModelIndexList selected;
    if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
        selected = ui->treeViewFiles->selectionModel()->selectedIndexes();
    else
        selected = ui->listViewFiles->selectionModel()->selectedIndexes ();

    if ( !selected.isEmpty() )
        m_listModel->setData( m_listModel->index(selected[0].row(),ListImagesModel::COLUMN_USER_ADJUST),offset.toPoint() );

    ui->widgetImageParameters->getScenePreview()->setSceneState(DesktopScene::STATE_IDLE);
    ui->graphicsViewDesktop->zoomToFit();

    // if previewed file and current wallpaper is the same => apply the modification
    if ( m_currentImageIndex == m_listModel->getIndex(m_currPreviewImage) )
        changeWallpaper(m_currentImageIndex,true);

    //restore state
    enableBrowsing(true);
}

void SyncWall::on_pushButtonUserCancel_pressed()
{
    int state = getState();

    // freeze interface if needed
    bool freeze = (state == STATE_CLIENT_CONNECTED ||
                   state == STATE_CLIENT_DISCONNECTED ||
                   state == STATE_CLIENT_INVALID);

    ui->pushButtonUserMove->setEnabled(ui->widgetImageParameters->isUserMoveAllowed() && !freeze);
    ui->pushButtonUserValid->setEnabled(false);
    ui->pushButtonUserCancel->setEnabled(false);

    ui->widgetImageParameters->getScenePreview()->setSceneState(DesktopScene::STATE_IDLE);
    ui->graphicsViewDesktop->zoomToFit();

    //restore state
    enableBrowsing(true);
}

void SyncWall::on_tabWidgetMain_currentChanged(int index)
{
    if (index == 0)
    {
        ui->widgetImageParameters->getScenePreview()->setSceneState(DesktopScene::STATE_IDLE);
        ui->graphicsViewDesktop->zoomToFit();
    }

    // just in case ...
    on_pushButtonUserCancel_pressed();
    ui->widgetScheduler->stopQuitAfterChange();
}

void SyncWall::on_tabWidgetPreferences_currentChanged(int index)
{
    if (index == 2)
    {
        ui->widgetImageParameters->getScenePreview()->setSceneState(DesktopScene::STATE_MOVING_SCREEN);
    }
}

void SyncWall::changeMonitorSupport()
{
    checkIfUserCanMove();

    // update preview offset
    QPoint offset = m_listModel->getOffset( m_listModel->getIndex(m_currPreviewImage) );
    ui->widgetImageParameters->setUserOffset(offset);
}

void SyncWall::checkIfUserCanMove()
{
    int state = getState();

    // freeze interface if needed
    bool freeze = (state == STATE_CLIENT_CONNECTED ||
                   state == STATE_CLIENT_DISCONNECTED ||
                   state == STATE_CLIENT_INVALID);

    if ( ui->widgetImageParameters->isUserMoveAllowed() && !freeze )
    {
        ui->pushButtonUserMove->setEnabled(true);
        ui->pushButtonUserValid->setEnabled(false);
        ui->pushButtonUserCancel->setEnabled(false);
    }
    else
    {
        ui->pushButtonUserMove->setEnabled(false);
        ui->pushButtonUserValid->setEnabled(false);
        ui->pushButtonUserCancel->setEnabled(false);
    }

    // update all the current screen number for list image
    m_listModel->setScreenNumber( ui->widgetImageParameters->getScreenNumberByConfig() );
}

void SyncWall::on_listWidgetTranslations_itemClicked( QListWidgetItem * item)
{
    QString lang = item->data(Qt::UserRole).toString();
    QString file( MiscFunctions::getTranslationsPath("fr") );
    file += QString("/%1_%2.qm").arg( QString(PACKAGE_NAME).toLower() ).arg(lang);

    QTranslator translator;
    translator.load(file);
    ui->labelTranslation->setText( translator.translate("","release of translation and translator name please","put your name here dear translator and the release of the translation file!!") );
}

// key events --------------------------------------------------------------------

void SyncWall::keyPressEvent(QKeyEvent* event)
{
    switch( event->key() )
    {
        case Qt::Key_Plus:
        {
            QModelIndexList selected;
            if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
                selected = ui->treeViewFiles->selectionModel()->selectedIndexes ();
            else
                selected = ui->listViewFiles->selectionModel()->selectedIndexes ();

            if ( !selected.isEmpty() )
            {
                QSet<int> doneRow;
                foreach (const QModelIndex &index, selected)
                {
                    if ( !doneRow.contains( index.row() ) )
                    {
                        int previousRate = m_listModel->getRating( index.row() );
                        m_listModel->setData( m_listModel->index(index.row(),ListImagesModel::COLUMN_RATE),++previousRate );
                        doneRow << index.row();
                    }
                }
            }
            break;
        }
        case Qt::Key_Minus:
        {
            QModelIndexList selected;
            if (m_listModel->getDisplayMode() != ListImagesModel::MODE_ICONS)
                selected = ui->treeViewFiles->selectionModel()->selectedIndexes ();
            else
                selected = ui->listViewFiles->selectionModel()->selectedIndexes ();

            if ( !selected.isEmpty() )
            {
                QSet<int> doneRow;
                foreach (const QModelIndex &index, selected)
                {
                    if ( !doneRow.contains( index.row() ) )
                    {
                        int previousRate = m_listModel->getRating( index.row() );
                        m_listModel->setData( m_listModel->index(index.row(),ListImagesModel::COLUMN_RATE),--previousRate );
                        doneRow << index.row();
                    }
                }
            }
            break;
        }
        default:
            break;
    }
    QMainWindow::keyPressEvent(event);
}

bool SyncWall::eventFilter(QObject* /*watched*/, QEvent* event)
{
    if (event->type() != QEvent::KeyPress)
        return false;

    QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
    keyPressEvent(keyEvent);
    return false;
}
