
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QFileInfo>
#include <QtCore/QDir>
#include <QtCore/QTextStream>
#include <QtCore/QSysInfo>
#include <QtCore/QDebug>

#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>

//#include <QtGui/QDesktopServices>

#include "SystemIntegration.h"

#include "AppSettings.h"
#include "LogHandler.h"
#include "MiscFunctions.h"

#ifdef Q_WS_WIN // for wallpaper set on windows platform
#include <windows.h>
#endif

#ifdef Q_WS_MAC
#include <strings.h>
#include <stdlib.h>
#include <sys/sysctl.h>
#endif

#ifdef Q_WS_PM
#define INCL_WINWORKPLACE
#include <os2.h>
#include <os2emx.h>
#endif

#if !defined(WIN32)
#include <unistd.h> // for unlink
#endif

SystemIntegration::SystemIntegration(QObject *parent)
    : QObject(parent),
    m_desktopType(DESKTOP_UNKNOWN)
{
    loadSettings();

    // force desktop type for known platform
#if defined(Q_WS_WIN)
    m_desktopType = DESKTOP_WINDOWS;
#elif defined(Q_WS_MAC)
    m_desktopType = DESKTOP_MACOSX;
#elif defined(Q_WS_PM)
    m_desktopType = DESKTOP_OS2;
#endif

    // not yet defined => try auto detect
    if (m_desktopType == DESKTOP_UNKNOWN)
        m_desktopType = detectCurrentDesktop();

    LogHandler::getInstance()->reportInfo( QString("Desktop selected: %1").arg( desktopToString(m_desktopType) ) );
    LogHandler::getInstance()->reportInfo( QString("Number of screen detected: %1").arg( QApplication::desktop()->screenCount() ) );

    // connect the process runner
    connect( &m_processRunner,SIGNAL( readyReadStandardOutput () ),this,SLOT( readStandardOutput() ) );
    connect( &m_processRunner,SIGNAL( readyReadStandardError () ),this,SLOT( readStandardError() ) );
    connect( &m_processRunner,SIGNAL( finished ( int, QProcess::ExitStatus) ),this,SLOT( finishedProcess( int, QProcess::ExitStatus) ) );
    connect( &m_processRunner,SIGNAL( error (QProcess::ProcessError) ),this,SLOT( errorProcess(QProcess::ProcessError) ) );

    // for debugging purpose displat the environment variables
    QProcessEnvironment currentEnv = QProcessEnvironment::systemEnvironment ();
    LogHandler::getInstance()->reportDebug("===================== begin system env =========================");
    foreach( const QString &env, currentEnv.toStringList() )
    LogHandler::getInstance()->reportDebug(env);
    LogHandler::getInstance()->reportDebug("===================== end system env =========================");

    updateDesktopTips();

    // clean temps storage
    QStringList fileFormats = MiscFunctions::getAvailablesImageFormats().split(" ");
    MiscFunctions::cleanTempStorage(fileFormats,"dynamic_wallpaper");
}

SystemIntegration::~SystemIntegration()
{
}

void SystemIntegration::applyImage(const QList<QImage> &imgs, bool keepWMSettings)
{
    QString bgImg;
    QString command;
    QStringList args;
    JobDef job;

    if ( imgs.isEmpty() )
        return;

    try
    {
        switch (m_desktopType)
        {
            case DESKTOP_WINDOWS:
            {
                bgImg = convertImage(imgs[0],"BMP");
#ifdef Q_WS_WIN
                QSettings appSettings( "HKEY_CURRENT_USER\\Control Panel\\Desktop", QSettings::NativeFormat);
                appSettings.setValue( "Wallpaper", QDir::toNativeSeparators(bgImg) );

                if (!keepWMSettings)
                {
                    /*
                       WallpaperStyle
                       0:  The image is centered if TileWallpaper=0 or tiled if TileWallpaper=1
                       2:  The image is stretched to fill the screen
                       6:  The image is resized to fit the screen while maintaining the aspect ratio. (Windows 7 and later)
                       10: The image is resized and cropped to fill the screen while maintaining the aspect ratio. (Windows 7 and later)
                     */
                    appSettings.setValue("WallpaperStyle","0");

                    /*
                       TileWallpaper
                       0: The wallpaper picture should not be tiled
                       1: The wallpaper picture should be tiled
                     */

                    appSettings.setValue("TileWallpaper","1");
                }

#ifdef UNICODE
                WCHAR wpath[1024];

                memset( wpath,0,1024 * sizeof(WCHAR) );
                QDir::toNativeSeparators(bgImg).toWCharArray(wpath);

                SystemParametersInfoW(SPI_SETDESKWALLPAPER,0,reinterpret_cast<void *> (wpath),SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
#else

                QByteArray ba = bgImg.toLatin1();
                SystemParametersInfoA(SPI_SETDESKWALLPAPER, 0, reinterpret_cast<void*>( ba.data() ),  SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
#endif

#endif
                break;
            }

            case DESKTOP_KDE3:
            {
                bgImg = convertImage(imgs[0],"PNG");

                //1 = none
                //2 = tiled
                //3 = center tiled
                //4 = centered
                //5 = scaled
                //6 = stretched
                QString wmode = "1";
                job.args << "kdesktop" << "KBackgroundIface" << "setWallpaper" << bgImg << wmode;
                job.command = "dcop";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_KDE4:
            {
                int screen = 0;
                foreach(const QImage &img, imgs)
                bgImg = convertImage(img,"PNG",screen++);

                //1 = none
                //2 = tiled
                //3 = center tiled
                //4 = centered
                //5 = scaled
                //6 = stretched
                QString wmode = "1";
                job.args << "kdesktop" << "KBackgroundIface" << "setWallpaper" << "1" << bgImg << wmode;
                job.command = "dbus";

                //m_cmdQueue.enqueue(job);
                LogHandler::getInstance()->reportWarning( QString("KDE4 is currently incompletely supported. Please select %1 as the current wallpaper with \"centered\" parameter (it's just a workaround)").arg(bgImg) );

                break;
            }

            case DESKTOP_GNOME:
            {
                bgImg = convertImage(imgs[0],"PNG");
                job.args << "--type" << "bool" << "--set" << "/desktop/gnome/background/draw_background" << "true";
                job.command = "gconftool-2";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "--type" << "string" << "--set" << "/desktop/gnome/background/picture_options" << "centered";
                job.command = "gconftool-2";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "--type" << "string" << "--set" << "/desktop/gnome/background/picture_filename" << bgImg;
                job.command = "gconftool-2";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_GNOME3:
            {
                bgImg = convertImage(imgs[0],"PNG");

                job.args << "set" << "org.gnome.desktop.background" << "draw-background" << "true";
                job.command = "gsettings";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "set" << "org.gnome.desktop.background" << "picture-options" << "spanned";
                job.command = "gsettings";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "set" << "org.gnome.desktop.background" << "picture-uri" << "\"file://" + bgImg + "\"";
                job.command = "gsettings";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_XFCE:
            {
                bgImg = convertImage(imgs[0],"PNG");
                QFile xfceFile(QDir::homePath() + "/.config/xfce4/desktop/backdrops.list");

                if ( !xfceFile.open(QIODevice::WriteOnly | QIODevice::Text) )
                {
                    LogHandler::getInstance()->reportError( QString("Can't open %1 ").arg( xfceFile.fileName() ) );
                    return;
                }

                xfceFile.open(QIODevice::WriteOnly);
                QTextStream out;
                out.setDevice(&xfceFile);

                out << "# xfce backdrop list" << endl;
                out << bgImg << endl;
                xfceFile.close();

                job.args << "--reload";
                job.command = "xfdesktop";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_OPENBOX:
            case DESKTOP_LXDE:
            {
                bgImg = convertImage(imgs[0],"PNG");

                // --desktop-pref                         Open desktop preference dialog
                // --set-wallpaper=<image file>           Set desktop wallpaper
                // --wallpaper-mode=<mode>                Set mode of desktop wallpaper.
                // <mode>=(color|stretch|fit|center|tile)

                job.args << "--wallpaper-mode=center";
                job.command = "pcmanfm";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "--set-wallpaper=" + bgImg;
                job.args << "--wallpaper-mode=fit"; // modif 19-04-2013 (margevicius.algimantas@gmail.com)
                job.command = "pcmanfm";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_XFCE4:
            {
                bgImg = convertImage(imgs[0],"PNG");

                job.args << "set" << "-c" << "xfce4-desktop" << "-p" << "/backdrop/screen0/monitor0/image-path" << "-s" << bgImg;
                job.command = "xfconf-query";
                m_cmdQueue.enqueue(job);
                job.args.clear();

                job.args << "set" << "-c" << "xfce4-desktop" << "-p" << "/backdrop/screen0/monitor0/image-style" << "-s" << "centered";
                job.command = "xfconf-query";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_FLUXBOX:
            {
                bgImg = convertImage(imgs[0],"PNG");

                //-f = stretched
                //-c = centered
                //-t = tiled
                job.args << "-c" << bgImg;
                job.command = "fbsetbg";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_BLACKBOX:
            {
                bgImg = convertImage(imgs[0],"PNG");

                //-full = stretched
                //-center = centered
                //-tile = tiled
                job.args << "-center" << bgImg;
                job.command = "bsetbg";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_ENLIGHTENMENT:
            {
                bgImg = convertImage(imgs[0],"PNG");

                // -c center
                job.args << "-c" << bgImg;
                job.command = "Esetroot";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_ICEWM:
            {
                bgImg = convertImage(imgs[0],"PNG");

                // use display from ImageMagick
                job.args << "-backdrop" << "-window" << "root" << bgImg;
                job.command = "display";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_WINDOWMAKER:
            {
                bgImg = convertImage(imgs[0],"PNG");

                //-t = tiled
                //-e = centered
                //-s = scaled
                job.args << "-e" << bgImg;
                job.command = "wmsetbg";
                m_cmdQueue.enqueue(job);
                break;
            }

            case DESKTOP_FVWM:
            {
                bgImg = convertImage(imgs[0],"PNG");
                job.args << bgImg;
                job.command = "fvwm-root";
                m_cmdQueue.enqueue(job);
                break;
            }

            case DESKTOP_MACOSX:
            {
                // extract from wally, a Qt4 wallpaper/background changer Copyright (C) 2009  Antonio Di Monaco <tony@becrux.com>
                // I've no mac so I can't test
                bgImg = convertImage(imgs[0],"PNG",0,true);

                QFile scriptFile(MiscFunctions::getValidStorage() + "/syncwall.scpt");
                QTextStream scriptText(&scriptFile);

                scriptFile.open(QIODevice::WriteOnly);
#ifdef Q_WS_MAC

                if (QSysInfo::MacintoshVersion == QSysInfo::MV_10_6)
                {
                    scriptText << "tell application \"System Events\"" << endl;
                    scriptText << "set picture of every desktop to POSIX file \"" << bgImg << "\"" << endl;
                    scriptText << "end tell" << endl;
                }
                else
                {
                    scriptText << "tell application \"Finder\"" << endl;
                    scriptText << "set desktop picture to POSIX file \"" << bgImg << "\"" << endl;
                    scriptText << "end tell" << endl;
                }

#endif
                scriptFile.close();

                job.args << QDir::toNativeSeparators(MiscFunctions::getValidStorage() + "/syncwall.scpt");
                job.command = "osascript";
                m_cmdQueue.enqueue(job);

                break;
            }

            case DESKTOP_OS2:
            {
                bgImg = convertImage(imgs[0],"BMP",0,true);
#ifdef Q_WS_PM
                HOBJECT hobj;
                const char *objName = "<WP_DESKTOP>";

                if ( hobj = WinQueryObject( reinterpret_cast< const unsigned char * > (objName) ) )
                {
                    WinSetObjectData( hobj,reinterpret_cast< const unsigned char * > ( ( QString("BACKGROUND=%1,S,1,I;").arg( QDir::toNativeSeparators(bgImg) ) ).toStdString().c_str() ) );
                }
                else
                {
                    LogHandler::getInstance()->reportDebug( QString ("Can't find object with name (%1)").arg(objName) );
                }

//          The first parameter after = is the file name, obviously.
//          Replace the 2nd parameter (N) with S or T to Scale or Tile.
//          The third parameter is the scale factor if needed.
//          The fourth parameter is I for image or C for colour.
//          The fifth parameter (only used for C and not shown here) is an RGB string e.g. 128 128 128
//
//		if (hobj = WinQueryObject("<WP_DESKTOP>"))
//			WinSetObjectData(hobj, "BACKGROUND=c:\\path\\image.bmp,N,1,I;");

#endif
                break;
            }

            default:
            {}
        }

        startNextJob();
    }
    catch (...)
    {
        LogHandler::getInstance()->reportError( QString("SystemIntegration::applyImage: Problem during saving of temp image %1!!").arg(bgImg) );
    }
}

QString SystemIntegration::convertImage(const QImage &img, const QString &format, int screenNum, bool roundrobin)
{
    static int count = 0;
    LogHandler::getInstance()->reportDebug( QString("Convert QImage to  %1").arg(format) );

    QString bgpath = QDir::toNativeSeparators( MiscFunctions::getValidStorage() + "/dynamic_wallpaper" + QString::number(screenNum) + "." + format.toLower() );

    if (roundrobin) // workaround for some wm (the file name must change each time)
    {
        if ( !m_lastCreatedFile.isEmpty() )
            unlink( m_lastCreatedFile.toStdString().c_str() );

        // add a "random" value to the filename
        m_lastCreatedFile = bgpath.replace( QString(".%1").arg( format.toLower() ),QString("-%1.%2").arg( qrand() ).arg( format.toLower() ) );
    }

    LogHandler::getInstance()->reportDebug( QString("Saving temp file (%1)").arg(bgpath) );

    if ( !img.save(bgpath) )
        LogHandler::getInstance()->reportInfo( QString("Failed to save temp file (%1)").arg(bgpath) );

    count++;
    return bgpath;
}

void SystemIntegration::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("SystemIntegration");

    m_desktopType = settings.value("desktopType",DESKTOP_UNKNOWN).toInt();

    settings.endGroup();
}

void SystemIntegration::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("SystemIntegration");

    settings.setValue("desktopType", m_desktopType);

    settings.endGroup();
}

QString SystemIntegration::desktopToString(int val)
{
    QString desktop;

    switch (val)
    {
        case DESKTOP_KDE3:
            desktop = QObject::tr("KDE3 desktop");
            break;

        case DESKTOP_KDE4:
            desktop = QObject::tr("KDE4 desktop");
            break;

        case DESKTOP_GNOME:
            desktop = QObject::tr("GNOME desktop");
            break;

        case DESKTOP_GNOME3:
            desktop = QObject::tr("GNOME3 desktop");
            break;

        case DESKTOP_XFCE:
            desktop = QObject::tr("XFCE desktop");
            break;

        case DESKTOP_XFCE4:
            desktop = QObject::tr("XFCE4 desktop");
            break;

        case DESKTOP_FLUXBOX:
            desktop = QObject::tr("FLUXBOX desktop");
            break;

        case DESKTOP_BLACKBOX:
            desktop = QObject::tr("BLACKBOX desktop");
            break;

        case DESKTOP_ENLIGHTENMENT:
            desktop = QObject::tr("Enlightenment desktop");
            break;

        case DESKTOP_ICEWM:
            desktop = QObject::tr("ICEWM desktop");
            break;

        case DESKTOP_OPENBOX:
            desktop = QObject::tr("OPENBOX desktop");
            break;

        case DESKTOP_LXDE:
            desktop = QObject::tr("LXDE desktop");
            break;

        case DESKTOP_WINDOWMAKER:
            desktop = QObject::tr("WINDOWMAKER desktop");
            break;

        case DESKTOP_FVWM:
            desktop = QObject::tr("FVWM desktop");
            break;

        case DESKTOP_WINDOWS:
            desktop = QObject::tr("WINDOWS desktop");
            break;

        case DESKTOP_MACOSX:
            desktop = QObject::tr("MACOSX desktop");
            break;

        case DESKTOP_OS2:
            desktop = QObject::tr("OS/2 desktop");
            break;

        case DESKTOP_UNKNOWN:
            desktop = QObject::tr("Automatic detection");
            break;

        default:
            desktop = QObject::tr("No Desktop Environment Detected");
            break;
    }

    return desktop;
}

//----------------------------------------------------------------
// detectCurrentDesktop is adapted from
// AutoWallpaper 2.1 (Linux) - version 2.1.2.x RC 1
// Tyler Jones - May 2009
//----------------------------------------------------------------

int SystemIntegration::detectCurrentDesktop()
{
    bool ok;
    desktopType temp(DESKTOP_UNKNOWN);

    // force desktop type for known platform
#if defined(Q_WS_WIN)
    temp = DESKTOP_WINDOWS;
#elif defined(Q_WS_MAC)
    temp = DESKTOP_MACOSX;
#elif defined(Q_WS_PM)
    temp = DESKTOP_OS2;
#endif

    QFileInfoList files = QDir("/proc").entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
    QListIterator<QFileInfo> file(files);
    int count = 0;

    while ( file.hasNext() && (temp == DESKTOP_UNKNOWN) )
    {
        if ( count > files.size() ) // sometimes this loop is infinite ????
        {
            LogHandler::getInstance()->reportError( QString("loop detected during environment processing => skip") );
            break;
        }

        file.peekNext().fileName().toInt(&ok);

        if (ok)
        {
            if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit") &&
                 !QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: KDE3") );
                temp = DESKTOP_KDE3;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: KDE4") );
                temp = DESKTOP_KDE4;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("gconfd-2") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: Gnome") );
                temp = DESKTOP_GNOME;

                // check if gnome 2 or 3
                QString version = MiscFunctions::launchExternalSynchronousProcess("gnome-session", QStringList() << "--version");
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Gnome version %1").arg(version) );

                if ( !version.isEmpty() && version.contains(" ") )
                {
                    QStringList list = version.split(" ");

                    if ( list.size() > 1 && list[1].startsWith("3") )
                        temp = DESKTOP_GNOME3;
                }
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("xfdesktop") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: Xfce") );
                temp = DESKTOP_XFCE;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("xfdesktop") ) // not yet functional !!!
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: Xfce4") );
                temp = DESKTOP_XFCE4;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("fluxbox") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: Fluxbox") );
                temp = DESKTOP_FLUXBOX;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("fvwm") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: FVWM") );
                temp = DESKTOP_FVWM;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("blackbox") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: Blackbox") );
                temp = DESKTOP_BLACKBOX;
            }
            else if ( QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("wmaker") )
            {
                LogHandler::getInstance()->reportDebug( QString("detectCurrentDesktop => Detected: WindowMaker") );
                temp = DESKTOP_WINDOWMAKER;
            }

            file.next();
        }
    }

    if (temp == DESKTOP_UNKNOWN)
        LogHandler::getInstance()->reportError( QString("No Desktop Environment Detected") );

    return temp;
}

void SystemIntegration::errorProcess( QProcess::ProcessError error)
{
    QString mess;

    switch (error)
    {
        case QProcess::FailedToStart:
        {
            mess = QString("SystemIntegration: The process failed to start. Either the invoked program is missing, or you may have insufficient permissions to invoke the program.");
            break;
        }

        case QProcess::Crashed:
        {
            mess = QString("SystemIntegration: The process crashed some time after starting successfully.");
            break;
        }

        case QProcess::Timedout:
        {
            mess = QString("SystemIntegration: The last waitFor...() function timed out. The state of QProcess is unchanged, and you can try calling waitFor...() again.");
            break;
        }

        case QProcess::WriteError:
        {
            mess = QString("SystemIntegration: An error occurred when attempting to write to the process. For example, the process may not be running, or it may have closed its input channel.");
            break;
        }

        case QProcess::ReadError:
        {
            mess = QString("SystemIntegration: An error occurred when attempting to read from the process. For example, the process may not be running.");
            break;
        }

        case QProcess::UnknownError:
        {
            mess = QString("SystemIntegration: An unknown error occurred.  This is the default return value of error().");
            break;
        }
    }

    LogHandler::getInstance()->reportError( mess );
    startNextJob();
}

void SystemIntegration::readStandardError()
{
    LogHandler::getInstance()->reportError( m_processRunner.readAllStandardError() );
}

void SystemIntegration::readStandardOutput()
{
    QString temp = m_processRunner.readAllStandardOutput();
    LogHandler::getInstance()->reportInfo(temp);
}

void SystemIntegration::finishedProcess( int /*exitCode*/, QProcess::ExitStatus exitStatus )
{
    switch (exitStatus)
    {
        case QProcess::NormalExit:
        {
            LogHandler::getInstance()->reportInfo( QString("The command exited normally.") );
            break;
        }

        case QProcess::CrashExit:
        {
            LogHandler::getInstance()->reportError( QString("The command crashed.") );
            break;
        }
    }

    startNextJob();
}

void SystemIntegration::startNextJob()
{
    // process already running
    if (m_processRunner.state () != QProcess::NotRunning)
    {
        LogHandler::getInstance()->reportDebug("process already running, waiting ...");
        return;
    }

    // no job in queue, exiting ...
    if ( m_cmdQueue.isEmpty() )
        return;

    m_currJob = m_cmdQueue.dequeue();
    LogHandler::getInstance()->reportDebug( QString("Running ") + m_currJob.command + " " + m_currJob.args.join(" ") );
    m_processRunner.start(m_currJob.command, m_currJob.args);
}

QString SystemIntegration::getDesktopTips(int index)
{
    if ( m_desktopTips.contains(index) )
        return m_desktopTips[index];
    else
        return "";
}

void SystemIntegration::updateDesktopTips()
{
    for (int i = DESKTOP_UNKNOWN; i < DESKTOP_TYPE_NUMBER; i++)
        m_desktopTips[i] = "";
    m_desktopTips[DESKTOP_KDE4] = QString("Tips: KDE4 is currently incompletely supported. Please select the generated wallpaper in plasma with \"centered\" parameter (it's just a workaround). For multi monitor with KDE activities, you can create a splitted wallpaper (one file by monitor) by selecting the right option in the \"Image parameters\" panel.");
}
