
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *******************************************************************************/

#include <QtCore/QTimer>
#include <QtNetwork/QTcpSocket>
#include <QtCore/QFile>

#include <QtCore/QtEndian>

#include "NetworkMessage.h"

#include "TcpClient.h"
#include "LogHandler.h"

const int defaultReconnectInterval = 20000; // 20s

//---------------------------------------------------------------------------------------------
TcpClient::TcpClient(QObject *parent) :
    QObject(parent),
    m_blockSize(0),
    m_tryToConnect(false),
    m_reconnect( new QTimer(this) )
{
    m_socket = new QTcpSocket(this);
    qRegisterMetaType<NetworkMessage>("NetworkMessage");
    qRegisterMetaType<QAbstractSocket::SocketState>("QAbstractSocket::SocketState");
    connect( m_socket, SIGNAL( connected() ),
             this, SLOT( onSocketConnected() ) );

    connect( m_socket, SIGNAL( error ( QAbstractSocket::SocketError ) ),
             this, SLOT( onSocketError(QAbstractSocket::SocketError) ) );

    connect( m_socket, SIGNAL( disconnected() ),
             this, SLOT( onSocketDisconnected() ) );

    connect( m_socket, SIGNAL( readyRead() ),
             this, SLOT( readMessage() ) );

    connect( m_socket, SIGNAL( stateChanged ( QAbstractSocket::SocketState) ),
             this, SIGNAL( stateChanged ( QAbstractSocket::SocketState) ) );

    m_reconnect->setInterval(defaultReconnectInterval);
    connect( m_reconnect,SIGNAL( timeout() ),this,SLOT( tryToConnect() ) );
}

//---------------------------------------------------------------------------------------------
TcpClient::~TcpClient()
{
    m_socket->abort();

    //delete m_socket;
}

//---------------------------------------------------------------------------------------------

void TcpClient::start()
{
    restart();
    m_tryToConnect = true;
}

void TcpClient::restart()
{
    stop();

    //m_connectionTimer->start();
    tryToConnect();
    m_reconnect->start();
}

void TcpClient::tryToConnect()
{
    LogHandler::getInstance()->reportError( QString("Try to connect to server %1.").arg( m_host ) );
    m_socket->connectToHost(m_host, m_port);
}

void TcpClient::stop()
{
//    m_connectionTimer->stop();
    m_socket->abort();
    m_tryToConnect = false;
    m_reconnect->stop();
}

//---------------------------------------------------------------------------------------------
void TcpClient::onSocketConnected()
{
    LogHandler::getInstance()->reportInfo( QString("Client: Connection to server established") );
    m_reconnect->stop();
}

//---------------------------------------------------------------------------------------------
void TcpClient::readMessage()
{
    // new block to read
    if(m_blockSize == 0)
    {
        if( m_socket->bytesAvailable() < (int)sizeof(qint64) )
            return;

        QByteArray buffer = m_socket->read( sizeof(qint64) );
        m_blockSize = qFromBigEndian<qint64>( (uchar*)buffer.constData() );
    }

    if(m_socket->bytesAvailable() < m_blockSize)
        return;

    m_buffer = m_socket->read(m_blockSize);

    emit newNetworkMessage( NetworkMessage(m_buffer) );
    m_blockSize = 0;

    // read waiting message if exists
    readMessage();
}

QString TcpClient::stateToString(QAbstractSocket::SocketState state)
{
    QString res;
    if(state == QAbstractSocket::UnconnectedState)
    {
        res = "Connection lost";
    }
    else if(state == QAbstractSocket::ConnectingState )
    {
        res = "Connecting";
    }
    else if(state == QAbstractSocket::ConnectedState)
    {
        res = "Connected";
    }
    else if(state == QAbstractSocket::HostLookupState)
    {
        res = "Performing host lookup";
    }
    else if(state == QAbstractSocket::ClosingState)
    {
        res = "Socket about to close";
    }
    return res;
}

//---------------------------------------------------------------------------------------------
void TcpClient::onSocketError(QAbstractSocket::SocketError)
{
    LogHandler::getInstance()->reportError( QString("Unable to connect: %1.").arg( m_socket->errorString() ) );
}

//---------------------------------------------------------------------------------------------
void TcpClient::onSocketDisconnected()
{
    LogHandler::getInstance()->reportError( QString("Client: Disconnect  from server ...") );

    if (m_tryToConnect)
        m_reconnect->start();
}

void TcpClient::sendNetworkMessage(const NetworkMessage &mess)
{
    QByteArray block( mess.toByteArray() );
    qint64 block_size = qToBigEndian<qint64>( block.size() );

    // write size
    m_socket->write( (const char*)&block_size,sizeof(qint64) );

    // write data
    m_socket->write(block);
    m_socket->flush();
}

QAbstractSocket::SocketState TcpClient::getState() const
{
    return m_socket->state();
}
