
#include <QtCore/QSettings>

#include <QtGui/QDesktopWidget>

#include <QtNetwork/QNetworkInterface>
#include <QtGui/QFileDialog>
#include <QtCore/QBuffer>

#include "mainwindowserver.h"
#include "ui_mainwindowserver.h"

#include "tcpserver.h"
#include "LogHandler.h"

const int defaultPort = 45454;

MainWindowServer::MainWindowServer(QWidget *parent) :
    QMainWindow(parent),
    m_ui(new Ui::MainWindowServer),
    m_server(NULL),
    m_tcpPort(defaultPort),
    m_autoStart(false)
{
    setWindowIcon( QIcon(":/syncwall/server.png") );
    m_ui->setupUi(this);
    loadSettings();

    // log
    connect( LogHandler::getInstance(),SIGNAL( newMessage(const QString &) ),this,SLOT( newMessage(const QString &) ) );
    LogHandler::getInstance()->setBufferization(false);

    prepareConnection();
}

MainWindowServer::~MainWindowServer()
{
    saveSettings();
    delete m_ui;
}

void MainWindowServer::on_pushButtonWaitForClient_toggled(bool val)
{
    setConnection(val);
}

void MainWindowServer::prepareConnection()
{
    // create tcpserver
    m_server = new TcpServer(this);
    connect( m_server,SIGNAL( stateChanged(quint8) ),this,SLOT( connectionChanged(quint8) ) );
    connect( m_server,SIGNAL( newNetworkMessage(const NetworkMessage &, const QString &) ),this,SLOT( newReceivedMessage(const NetworkMessage &, const QString &) ) );
    connectionChanged(TcpServer::INACTIVE);

    // Update UI
    QString ip;
    foreach ( const QHostAddress &address, QNetworkInterface::allAddresses() )
    {
        if ( address.protocol() == QAbstractSocket::IPv4Protocol && address != QHostAddress(QHostAddress::LocalHost && address.toString().section( ".",-1,-1 ) != "1") )
        {
            ip = address.toString();
            break;
        }
    }
    m_ui->labelServerIp->setText(ip);
}

void MainWindowServer::setConnection(bool val)
{
    if (val)
    {
        m_server->start();
    }
    else
    {
        m_server->stop();
    }
}

void MainWindowServer::connectionChanged(quint8 new_state)
{
    m_ui->labelConnectionState->setText( m_server->getStateAsString(new_state) );

    switch(new_state)
    {
        case TcpServer::INACTIVE:
        {
            m_ui->labelConnectionStatePixmap->setPixmap( QPixmap(":/syncwall/disconnected.png") );
            break;
        }
        case TcpServer::LISTENING:
        {
            m_ui->labelConnectionStatePixmap->setPixmap( QPixmap(":/syncwall/waiting.png") );
            break;
        }
        case TcpServer::CONNECTED:
        {
            m_ui->labelConnectionStatePixmap->setPixmap( QPixmap(":/syncwall/connected.png") );
            break;
        }
        default:
            break;
    }
    m_ui->labelConnected->setText( tr("Connected: %1").arg( m_server->getConnectionNumber() ) );
}

void MainWindowServer::on_pushButtonSendMessage_pressed()
{
    NetworkMessage mess;

    //mess.id = m_ui->spinBoxSendId->value();
    m_lastString = m_ui->lineEditSendValue->text();
    mess.options = m_lastString;
    m_server->sendNetworkMessage(mess);
}

void MainWindowServer::newReceivedMessage(const NetworkMessage &mess, const QString &client)
{
    m_ui->plainTextEditReceivedMsg->clear();
    m_ui->plainTextEditReceivedMsg->appendPlainText( QString("From: %1 => ").arg(client) + mess.options );
}

//-----------------------------------------------------------------------
// Save/Restore parameters functions
//-----------------------------------------------------------------------

void MainWindowServer::saveSettings()
{
    QSettings settings("MySoft", "MainWindowServer");

    settings.beginGroup("MainWindow");

    settings.setValue( "MainWindowState",saveState(0) );

    settings.setValue( "size", size() );
    settings.setValue( "pos", pos() );

    settings.setValue( "fullScreen", isFullScreen() );

    // save the screen number for the splashsreeen ...
    if ( QApplication::desktop()->isVirtualDesktop() )
        settings.setValue( "screenNumber", QApplication::desktop()->screenNumber( pos() ) );
    else
        settings.setValue( "screenNumber", QApplication::desktop()->screenNumber(this) );

    settings.setValue( "screenNumber", QApplication::desktop()->screenNumber(this) );

    settings.endGroup();

    settings.beginGroup("Application");

    settings.setValue("tcpPort", m_tcpPort);

    settings.setValue("autoStart", m_autoStart);

    settings.setValue("lastImage", m_lastImage);
    settings.setValue("lastString", m_lastString);

    settings.endGroup();
}

void MainWindowServer::loadSettings()
{
    QSettings settings("MySoft", "MainWindowServer");

    settings.beginGroup("MainWindow");

    restoreState(settings.value("MainWindowState").toByteArray(), 0);

    resize( settings.value( "size", QSize(500, 500) ).toSize() );
    move( settings.value( "pos", QPoint(200, 200) ).toPoint() );

    bool fullScreen = settings.value("fullScreen",false).toBool();
    if (fullScreen)
        showFullScreen ();

    settings.endGroup();

    settings.beginGroup("Application");

    m_tcpPort = settings.value("tcpPort",defaultPort).toInt();
    m_ui->spinBoxUdpPort->setValue(m_tcpPort);

    m_autoStart = settings.value("autoStart",defaultPort).toBool();
    if (m_autoStart)
        m_ui->pushButtonAutoStart->animateClick();

    m_lastImage = settings.value("lastImage","").toString();
    if ( !m_lastImage.isEmpty() )
        previewImage(m_lastImage);

    m_lastString = settings.value("lastString","").toString();
    m_ui->lineEditSendValue->setText(m_lastString);

    settings.endGroup();
}

void MainWindowServer::send(const NetworkMessage &msg)
{
}

void MainWindowServer::on_pushButtonQuit_pressed()
{
    QApplication::quit();
}

void MainWindowServer::on_spinBoxUdpPort_valueChanged(int val)
{
    m_tcpPort = val;
    m_server->setPort(m_tcpPort);
}

void MainWindowServer::newMessage(const QString &mess)
{
    m_ui->listWidgetLog->addItem(mess);
    m_ui->listWidgetLog->scrollToItem( m_ui->listWidgetLog->item(m_ui->listWidgetLog->count() - 1) );
}

void MainWindowServer::on_pushButtonAutoStart_toggled(bool val)
{
    m_autoStart = val;
    if ( m_autoStart && !m_ui->pushButtonWaitForClient->isChecked() )
        m_ui->pushButtonWaitForClient->animateClick();
}

void MainWindowServer::on_pushButtonImage_pressed()
{
    QString file = QFileDialog::getOpenFileName(
        this,
        "Select one image to send",
        m_lastImage,
        "*.png *.jpg");

    if ( !file.isEmpty() )
    {
        previewImage(file);
        return;
    }
}

void MainWindowServer::previewImage(const QString &file)
{
    m_lastImage = file;
    m_ui->lineEditImage->setText(m_lastImage);
    m_ui->labelPreview->setPixmap( QPixmap(m_lastImage).scaled(QSize(48,48),Qt::KeepAspectRatio) );
}

void MainWindowServer::on_pushButtonSendImage_pressed()
{
    if ( m_lastImage.isEmpty() )
        return;

    // prepare message

    QImage thumb(m_lastImage );
    /*
    QByteArray bytes;
    QBuffer buffer(&bytes);
    buffer.open(QIODevice::WriteOnly);
    thumb.save(&buffer, "PNG");
    */

    NetworkMessage mess;
    mess.options = m_lastString;
    mess.image = thumb;
    m_server->sendNetworkMessage(mess);
}
